@@ -31,10 +31,12 @@
 KoWmfPaint::KoWmfPaint()
     : KoWmfRead()
     , mTextPen()
+    , mSaveCount(0)
 {
     mTarget = 0;
     mIsInternalPainter = true;
     mPainter = 0;
+    mWorldTransform = QTransform();
 }
 
 
@@ -49,7 +51,13 @@ bool KoWmfPaint::play(QPaintDevice& target, bool relativeCoord)
     mRelativeCoord = relativeCoord;
 
     // Play the wmf file
-    return KoWmfRead::play();
+    mSaveCount = 0;
+    bool ret = KoWmfRead::play();
+
+    // Make sure that the painter is in the same state as before KoWmfRead::play()
+    for (; mSaveCount > 0; mSaveCount--)
+        restore();
+    return ret;
 }
 
 bool KoWmfPaint::play(QPainter &painter, bool relativeCoord)
@@ -66,7 +74,13 @@ bool KoWmfPaint::play(QPainter &painter, bool relativeCoord)
     mRelativeCoord = relativeCoord;
 
     // Play the wmf file
-    return KoWmfRead::play();
+    mSaveCount = 0;
+    bool ret = KoWmfRead::play();
+
+    // Make sure that the painter is in the same state as before KoWmfRead::play()
+    for (; mSaveCount > 0; mSaveCount--)
+        restore();
+    return ret;
 }
 
 
@@ -75,36 +89,30 @@ bool KoWmfPaint::play(QPainter &painter, bool relativeCoord)
 
 bool KoWmfPaint::begin()
 {
-    bool ret = true;
-
     // If the painter is our own, we have to call begin() on it.
-    // If it's external, we suppose that it's already done for us.
-    if (mIsInternalPainter)
-        ret = mPainter->begin(mTarget);
-
-    if (ret) {
-        if (mRelativeCoord) {
-            mInternalWorldMatrix.reset();
-
-            // This window is used for scaling in setWindowExt().
-            mPainter->setWindow(boundingRect());
-        } else {
-            // Some WMF files don't call setwindowOrg and
-            // setWindowExt, so it's better to do it here.  Note that
-            // boundingRect() is the rect of the WMF file, not the device.
-            QRect rec = boundingRect();
-            //kDebug(31000) << "BoundingRect: " << rec;
-            mPainter->setWindow(rec);
-        }
+    // If it's external, we assume that it's already done for us.
+    if (mIsInternalPainter) {
+        if (!mPainter->begin(mTarget))
+            return false;
     }
 
+    // For calculations of window / viewport during the painting
+    mWindowOrg = QPoint(0, 0);
+    mViewportOrg = QPoint(0, 0);
+    mWindowExtIsSet = false;
+    mViewportExtIsSet = false;
+    mWindowViewportIsSet = false;
+    mOutputTransform = mPainter->transform();
+    mWorldTransform = QTransform();
+
+    mPainter->setBrush(QBrush(Qt::NoBrush));
+
 #if DEBUG_WMFPAINT
     kDebug(31000) << "Using QPainter: " << mPainter->pen() << mPainter->brush() 
                   << "Background: " << mPainter->background() << " " << mPainter->backgroundMode();
-    kDebug(31000) << "KoWmfPaint::begin returns " << ret;
 #endif
 
-    return ret;
+    return true;
 }
 
 
@@ -120,13 +128,36 @@ bool KoWmfPaint::end()
 
 void KoWmfPaint::save()
 {
+    // A little trick here: Save the worldTransform in the painter.
+    // If we didn't do this, we would have to create a separate stack
+    // for these.
+    //
+    // FIXME: We should collect all the parts of the DC that are not
+    //        stored in the painter and save them separately.
+    QTransform  savedTransform = mPainter->worldTransform();
+    mPainter->setWorldTransform(mWorldTransform);
+
     mPainter->save();
+    ++mSaveCount;
+
+    mPainter->setWorldTransform(savedTransform);
 }
 
 
 void KoWmfPaint::restore()
 {
-    mPainter->restore();
+    if (mSaveCount > 0) {
+        mPainter->restore();
+        mSaveCount--;
+    }
+    else {
+        kDebug(31000) << "restore(): try to restore painter without save";
+    }
+
+    // We used a trick in save() and stored the worldTransform in
+    // the painter.  Now restore the full transformation.
+    mWorldTransform = mPainter->worldTransform();
+    recalculateWorldTransform();
 }
 
 
@@ -235,6 +266,32 @@ void KoWmfPaint::setCompositionMode(QPainter::CompositionMode mode)
 
 
 // ---------------------------------------------------------------------
+//                 World Transform, Window and Viewport
+
+
+// General note about coordinate spaces and transforms:
+//
+// There are several coordinate spaces in use when drawing an EMF file:
+//  1. The object space, in which the objects' coordinates are expressed inside the EMF.
+//     In general there are several of these.
+//  2. The page space, which is where they end up being painted in the EMF picture.
+//     The union of these form the bounding box of the EMF.
+//  3. (possibly) the output space, where the EMF picture itself is placed
+//     and/or scaled, rotated, etc
+//
+// The transform between spaces 1. and 2. is called the World Transform.
+// The world transform can be changed either through calls to change
+// the window or viewport or through calls to setWorldTransform() or
+// modifyWorldTransform().
+//
+// The transform between spaces 2. and 3. is the transform that the QPainter
+// already contains when it is given to us.  We need to save this and reapply
+// it after the world transform has changed. We call this transform the Output
+// Transform in lack of a better word. (Some sources call it the Device Transform.)
+//
+
+
+// FIXME:
 // To change those functions it's better to have
 // a large set of WMF files. WMF special case includes :
 // - without call to setWindowOrg and setWindowExt
@@ -242,32 +299,118 @@ void KoWmfPaint::setCompositionMode(QPainter::CompositionMode mode)
 // - negative width or height
 // and relative/absolute coordinate
 
-void KoWmfPaint::setWindowOrg(int orgX, int orgY) // Love that parameter name...
+
+void KoWmfPaint::recalculateWorldTransform()
+{
+    mWorldTransform = QTransform();
+
+    if (!mWindowExtIsSet && !mViewportExtIsSet)
+        return;
+
+    // FIXME: Check windowExt == 0 in any direction
+    if (mWindowExtIsSet && mViewportExtIsSet) {
+        // Both window and viewport are set.
+        mWindowViewportScaleX = qreal(mViewportExt.width()) / qreal(mWindowExt.width());
+        mWindowViewportScaleY = qreal(mViewportExt.height()) / qreal(mWindowExt.height());
+#if DEBUG_WMFPAINT
+        kDebug(31000) << "Scale for Window -> Viewport"
+                      << mWindowViewportScaleX << mWindowViewportScaleY;
+#endif
+    } else {
+        // Only one of window and viewport ext is set: Use same width for window and viewport
+        mWindowViewportScaleX = qreal(1.0);
+        mWindowViewportScaleY = qreal(1.0);
+#if DEBUG_WMFPAINT
+        kDebug(31000) << "Only one of Window or Viewport set: scale = 1";
+#endif
+    }
+
+    // Negative window extensions mean flip the picture.  Handle this here.
+    bool  flip = false;
+    qreal midpointX = 0.0;
+    qreal midpointY = 0.0;
+    qreal scaleX = 1.0;
+    qreal scaleY = 1.0;
+    if (mWindowExt.width() < 0) {
+        midpointX = mWindowOrg.x() + mWindowExt.width() / qreal(2.0);
+        scaleX = -1.0;
+        flip = true;
+    }
+    if (mWindowExt.height() < 0) {
+        midpointY = mWindowOrg.y() + mWindowExt.height() / qreal(2.0);
+        scaleY = -1.0;
+        flip = true;
+    }
+    if (flip) {
+        //kDebug(31000) << "Flipping" << midpointX << midpointY << scaleX << scaleY;
+        mWorldTransform.translate(midpointX, midpointY);
+        mWorldTransform.scale(scaleX, scaleY);
+        mWorldTransform.translate(-midpointX, -midpointY);
+        //kDebug(31000) << "After flipping for window" << mWorldTransform;
+    }
+
+    // Calculate the world transform.
+    mWorldTransform.translate(-mWindowOrg.x(), -mWindowOrg.y());
+    mWorldTransform.scale(mWindowViewportScaleX, mWindowViewportScaleY);
+    if (mViewportExtIsSet) {
+        mWorldTransform.translate(mViewportOrg.x(), mViewportOrg.y());
+    } 
+    else {
+        mWorldTransform.translate(mWindowOrg.x(), mWindowOrg.y());
+    }
+    //kDebug(31000) << "After window viewport calculation" << mWorldTransform;
+
+    // FIXME: also handle negative viewport extensions?  If so, do it here.
+
+    // Apply the world transform to the painter.
+    mPainter->setWorldTransform(mWorldTransform);
+    mWindowViewportIsSet = true;
+
+    // Apply the output transform.
+    QTransform currentMatrix = mPainter->worldTransform();
+    QTransform newMatrix = currentMatrix * mOutputTransform;
+    //kDebug(31000) << "Output transform" << mOutputTransform;
+    //kDebug(31000) << "Total  transform" << newMatrix;
+    mPainter->setWorldTransform( newMatrix );
+}
+
+
+
+
+void KoWmfPaint::setWindowOrg(int left, int top)
 {
 #if DEBUG_WMFPAINT
-    kDebug(31000) << orgX << " " << orgY;
+    kDebug(31000) << left << " " << top;
 #endif
 
-    mOrgX = orgX;
-    mOrgY = orgY;
+    mWindowOrg = QPoint(left, top);
 
-    if (mRelativeCoord) {
-        // Translate back from last translation to the origin.
-        qreal dx = mInternalWorldMatrix.dx();
-        qreal dy = mInternalWorldMatrix.dy();
-        //kDebug(31000) << "old translation: " << dx << dy;
-        //kDebug(31000) << mInternalWorldMatrix;
-        //kDebug(31000) << "new translation: " << -orgX << -orgY;
-        mInternalWorldMatrix.translate(-dx, -dy);
-        mPainter->translate(-dx, -dy);
+    recalculateWorldTransform();
 
-        // Translate to the new origin.
-        mInternalWorldMatrix.translate(-orgX, -orgY);
-        mPainter->translate(-orgX, -orgY);
-    } else {
-        QRect rec = mPainter->window();
-        mPainter->setWindow(orgX, orgY, rec.width(), rec.height());
-    }
+#if 0
+    // Debug code.  Draw a rectangle with some decoration to show see
+    // if all the transformations work.
+    mPainter->save();
+
+    // Paint a black rectangle around the current window.
+    mPainter->setPen(Qt::green);
+    QRect windowRect = QRect(mWindowOrg, mWindowExt);
+    mPainter->drawRect(windowRect);
+
+#if 0
+    // Paint a black line from the Window origin to (0, 0)
+    mPainter->drawLine(mWindowOrg, QPoint(0, 0));
+
+    mPainter->setPen(Qt::red);
+    mPainter->drawRect(boundingRect());
+
+    mPainter->drawLine(boundingRect().topLeft(), QPoint(0, 0));
+#endif
+    mPainter->restore();
+
+    kDebug(31000) << "Window rect: " << windowRect;
+    kDebug(31000) << "Bounding rect: " << boundingRect();
+#endif
 }
 
 
@@ -277,45 +420,58 @@ void KoWmfPaint::setWindowExt(int width, int height)
     kDebug(31000) << width << " " << height;
 #endif
 
-    mExtWidth = width;
-    mExtHeight = height;
+    mWindowExt = QSize(width, height);
+    mWindowExtIsSet = true;
 
-    if (mRelativeCoord) {
-        QRect r = mPainter->window();
-        qreal dx = mInternalWorldMatrix.dx();
-        qreal dy = mInternalWorldMatrix.dy();
-        qreal sx = mInternalWorldMatrix.m11();
-        qreal sy = mInternalWorldMatrix.m22();
+    recalculateWorldTransform();
 
-        // Translate and scale back from the last window.
-        mInternalWorldMatrix.translate(-dx, -dy);
-        mPainter->translate(-dx, -dy);
-        mInternalWorldMatrix.scale(1 / sx, 1 / sy);
-        mPainter->scale(1 / sx, 1 / sy);
-
-        sx = (qreal)r.width()  / (qreal)width;
-        sy = (qreal)r.height() / (qreal)height;
-
-        // Scale and translate into the new window
-        mInternalWorldMatrix.scale(sx, sy);
-        mPainter->scale(sx, sy);
-        mInternalWorldMatrix.translate(dx, dy);
-        mPainter->translate(dx, dy);
-    } else {
-        QRect rec = mPainter->window();
-        mPainter->setWindow(rec.left(), rec.top(), width, height);
-    }
 #if 0
+    // Debug code.  Draw a rectangle with some decoration to show see
+    // if all the transformations work.
     mPainter->save();
-    mPainter->setPen(Qt::black);
-    mPainter->drawRect(QRect(QPoint(mOrgX, mOrgY),
-                             QSize(mExtWidth, mExtHeight)));
+
+    // Paint a red rectangle around the current window.
     mPainter->setPen(Qt::red);
+    QRect windowRect = QRect(mWindowOrg, mWindowExt);
+    mPainter->drawRect(windowRect);
+
+    // Paint a line from the Window origin to (0, 0)
+    mPainter->drawLine(mWindowOrg, QPoint(0, 0));
+
+    mPainter->setPen(Qt::black);
     mPainter->drawRect(boundingRect());
+
+    mPainter->drawLine(boundingRect().topLeft(), QPoint(0, 0));
     mPainter->restore();
+
+    kDebug(31000) << "Window rect: " << windowRect;
+    kDebug(31000) << "Bounding rect: " << boundingRect();
 #endif
 }
 
+void KoWmfPaint::setViewportOrg( int left, int top )
+{
+#if DEBUG_WMFPAINT
+    kDebug(31000) << left << top;
+#endif
+
+    mViewportOrg = QPoint(left, top);
+
+    recalculateWorldTransform();
+}
+
+void KoWmfPaint::setViewportExt( int width, int height )
+{
+#if DEBUG_WMFPAINT
+    kDebug(31000) << width << height;
+#endif
+
+    mViewportExt = QSize(width, height);
+    mViewportExtIsSet = true;
+
+    recalculateWorldTransform();
+}
+
 
 void KoWmfPaint::setMatrix(const QMatrix &wm, bool combine)
 {
@@ -323,9 +479,15 @@ void KoWmfPaint::setMatrix(const QMatrix &wm, bool combine)
     kDebug(31000) << wm << " " << combine;
 #endif
     mPainter->setMatrix(wm, combine);
+
+    recalculateWorldTransform();
 }
 
 
+// ----------------------------------------------------------------
+//                         Drawing
+
+
 void KoWmfPaint::setClipRegion(const QRegion &rec)
 {
     mPainter->setClipRegion(rec);
@@ -521,6 +683,9 @@ void KoWmfPaint::drawText(int x, int y, int w, int h, int textAlign, const QStri
         // (left, top) position = current logical position
         x = mLastPos.x();
         y = mLastPos.y();
+#if DEBUG_WMFPAINT
+        kDebug(31000) << "Using current position:" << x << y;
+#endif
     }
 
     QFontMetrics  fm(mPainter->font(), mTarget);
@@ -538,13 +703,13 @@ void KoWmfPaint::drawText(int x, int y, int w, int h, int textAlign, const QStri
         y -= fm.ascent();  // (height - fm.descent()) is used in qwmf.  This should be the same.
     else if ((textAlign & TA_BOTTOM) == TA_BOTTOM) {
         y -= height;
+    }
 
 #if DEBUG_WMFPAINT
-        kDebug(31000) << "font = " << mPainter->font() << " pointSize = " << mPainter->font().pointSize()
-                      << "ascent = " << fm.ascent() << " height = " << fm.height()
-                      << "leading = " << fm.leading();
+    kDebug(31000) << "font = " << mPainter->font() << " pointSize = " << mPainter->font().pointSize()
+                  << "ascent = " << fm.ascent() << " height = " << fm.height()
+                  << "leading = " << fm.leading();
 #endif
-    }
 
     // Use the special pen defined by mTextPen for text.
     QPen  savePen = mPainter->pen();
