@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2005 Jaroslaw Staniek <js@iidea.pl>
+   Copyright (C) 2005 Jarosław Staniek <staniek@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -22,7 +22,6 @@
 #include <kaction.h>
 
 KexiSharedActionClient::KexiSharedActionClient()
- : m_sharedActions(101, false)
 {
 }
 
@@ -30,10 +29,10 @@ KexiSharedActionClient::~KexiSharedActionClient()
 {
 }
 
-void KexiSharedActionClient::plugSharedAction(KAction* a)
+void KexiSharedActionClient::plugSharedAction(QAction* a)
 {
-	if (!a)
-		return;
-	m_sharedActions.insert(a->name(), a);
+    if (!a)
+        return;
+    m_sharedActions.insert(a->objectName(), a);
 }
 
