@@ -122,10 +122,11 @@ void MacroCommand::unexecute()
 }
 
 //-------------------------------------------------
-CalendarAddCmd::CalendarAddCmd( Project *project, Calendar *cal, Calendar *parent, const QString& name )
+CalendarAddCmd::CalendarAddCmd( Project *project, Calendar *cal, int pos, Calendar *parent, const QString& name )
         : NamedCommand( name ),
         m_project( project ),
         m_cal( cal ),
+        m_pos( pos ),
         m_parent( parent ),
         m_mine( true )
 {
@@ -140,7 +141,7 @@ CalendarAddCmd::~CalendarAddCmd()
 void CalendarAddCmd::execute()
 {
     if ( m_project ) {
-        m_project->addCalendar( m_cal, m_parent );
+        m_project->addCalendar( m_cal, m_parent, m_pos );
         m_mine = false;
     }
 
@@ -162,14 +163,14 @@ CalendarRemoveCmd::CalendarRemoveCmd( Project *project, Calendar *cal, const QSt
         m_project( project ),
         m_parent( cal->parentCal() ),
         m_cal( cal ),
+        m_index( -1 ),
         m_mine( false ),
         m_cmd( new MacroCommand("") )
 {
     Q_ASSERT( project != 0 );
 
-/*    foreach ( Schedule * s, project->schedules() ) {
-        addSchScheduled( s );
-    }*/
+    m_index = m_parent ? m_parent->indexOf( cal ) : project->indexOf( cal );
+
     foreach ( Resource *r, project->resourceList() ) {
         if ( r->calendar( true ) == cal ) {
             m_cmd->addCommand( new ModifyResourceCalendarCmd( r, 0 ) );
@@ -199,13 +200,38 @@ void CalendarRemoveCmd::execute()
 }
 void CalendarRemoveCmd::unexecute()
 {
-    m_project->addCalendar( m_cal, m_parent );
+    m_project->addCalendar( m_cal, m_parent, m_index );
     m_cmd->unexecute();
 //    setSchScheduled();
     m_mine = false;
 
 }
 
+CalendarMoveCmd::CalendarMoveCmd( Project *project, Calendar *cal, int position, Calendar *parent, const QString& name )
+        : NamedCommand( name ),
+        m_project( project ),
+        m_cal( cal ),
+        m_newpos( position ),
+        m_newparent( parent ),
+        m_oldparent( cal->parentCal() )
+{
+    //kDebug()<<cal->name();
+    Q_ASSERT( project != 0 );
+
+    m_oldpos = m_oldparent ? m_oldparent->indexOf( cal ) : project->indexOf( cal );
+}
+void CalendarMoveCmd::execute()
+{
+    m_project->takeCalendar( m_cal );
+    m_project->addCalendar( m_cal, m_newparent, m_newpos );
+}
+
+void CalendarMoveCmd::unexecute()
+{
+    m_project->takeCalendar( m_cal );
+    m_project->addCalendar( m_cal, m_oldparent, m_oldpos );
+}
+
 CalendarModifyNameCmd::CalendarModifyNameCmd( Calendar *cal, const QString& newvalue, const QString& name )
         : NamedCommand( name ),
         m_cal( cal )
@@ -232,10 +258,14 @@ CalendarModifyParentCmd::CalendarModifyParentCmd( Project *project, Calendar *ca
         : NamedCommand( name ),
         m_project( project ),
         m_cal( cal ),
-        m_cmd( new MacroCommand( "" ) )
+        m_cmd( new MacroCommand( "" ) ),
+        m_oldindex( -1 ),
+        m_newindex( -1 )
 {
     m_oldvalue = cal->parentCal();
     m_newvalue = newvalue;
+    m_oldindex = m_oldvalue ? m_oldvalue->indexOf( cal ) : m_project->indexOf( cal );
+
     if ( newvalue ) {
         m_cmd->addCommand( new CalendarModifyTimeZoneCmd( cal, newvalue->timeZone() ) );
     }
@@ -248,7 +278,7 @@ CalendarModifyParentCmd::~CalendarModifyParentCmd()
 void CalendarModifyParentCmd::execute()
 {
     m_project->takeCalendar( m_cal );
-    m_project->addCalendar( m_cal, m_newvalue );
+    m_project->addCalendar( m_cal, m_newvalue, m_newindex );
     m_cmd->execute();
 //    setSchScheduled( false );
 
@@ -257,7 +287,7 @@ void CalendarModifyParentCmd::unexecute()
 {
     m_cmd->unexecute();
     m_project->takeCalendar( m_cal );
-    m_project->addCalendar( m_cal, m_oldvalue );
+    m_project->addCalendar( m_cal, m_oldvalue, m_oldindex );
 //    setSchScheduled();
 
 }
@@ -414,7 +444,7 @@ CalendarModifyStateCmd::CalendarModifyStateCmd( Calendar *calendar, CalendarDay
     m_newvalue = value;
     m_oldvalue = (CalendarDay::State)day->state();
     if ( value != CalendarDay::Working ) {
-        foreach ( TimeInterval *ti, day->workingIntervals() ) {
+        foreach ( TimeInterval *ti, day->timeIntervals() ) {
             m_cmd->addCommand( new CalendarRemoveTimeIntervalCmd( calendar, day, ti ) );
         }
     }
@@ -1945,6 +1975,36 @@ void ModifyRequiredResourcesCmd::unexecute()
     m_resource->setRequiredResources( m_oldvalue );
 }
 
+AddResourceTeamCmd::AddResourceTeamCmd( Resource *team, Resource *member, const QString& name )
+    : NamedCommand( name ),
+    m_team( team ),
+    m_member( member )
+{
+}
+void AddResourceTeamCmd::execute()
+{
+    m_team->addTeamMember( m_member );
+}
+void AddResourceTeamCmd::unexecute()
+{
+    m_team->removeTeamMember( m_member );
+}
+
+RemoveResourceTeamCmd::RemoveResourceTeamCmd( Resource *team, Resource *member, const QString& name )
+    : NamedCommand( name ),
+    m_team( team ),
+    m_member( member )
+{
+}
+void RemoveResourceTeamCmd::execute()
+{
+    m_team->removeTeamMember( m_member );
+}
+void RemoveResourceTeamCmd::unexecute()
+{
+    m_team->addTeamMember( m_member );
+}
+
 RemoveResourceGroupCmd::RemoveResourceGroupCmd( Project *project, ResourceGroup *group, const QString& name )
         : NamedCommand( name ),
         m_group( group ),
@@ -2382,21 +2442,23 @@ void AddCompletionActualEffortCmd::unexecute()
 
 }
 
-AddAccountCmd::AddAccountCmd( Project &project, Account *account, const QString& parent, const QString& name )
+AddAccountCmd::AddAccountCmd( Project &project, Account *account, const QString& parent, int index, const QString& name )
         : NamedCommand( name ),
         m_project( project ),
         m_account( account ),
         m_parent( 0 ),
+        m_index( index ),
         m_parentName( parent )
 {
     m_mine = true;
 }
 
-AddAccountCmd::AddAccountCmd( Project &project, Account *account, Account *parent, const QString& name )
+AddAccountCmd::AddAccountCmd( Project &project, Account *account, Account *parent, int index, const QString& name )
         : NamedCommand( name ),
         m_project( project ),
         m_account( account ),
-        m_parent( parent )
+        m_parent( parent ),
+        m_index( index )
 {
     m_mine = true;
 }
@@ -2412,7 +2474,7 @@ void AddAccountCmd::execute()
     if ( m_parent == 0 && !m_parentName.isEmpty() ) {
         m_parent = m_project.accounts().findAccount( m_parentName );
     }
-    m_project.accounts().insert( m_account, m_parent );
+    m_project.accounts().insert( m_account, m_parent, m_index );
 
 
     m_mine = false;
@@ -2763,12 +2825,12 @@ void ProjectModifyEndTimeCmd::unexecute()
 }
 
 //----------------------------
-AddScheduleManagerCmd::AddScheduleManagerCmd( Project &node, ScheduleManager *sm, const QString& name )
+AddScheduleManagerCmd::AddScheduleManagerCmd( Project &node, ScheduleManager *sm, int index, const QString& name )
     : NamedCommand( name ),
     m_node( node ),
     m_parent( sm->parentManager() ),
     m_sm( sm ),
-    m_index( -1 ),
+    m_index( index ),
     m_exp( sm->expected() ),
     m_opt( sm->optimistic() ),
     m_pess( sm->pessimistic() ),
@@ -2776,12 +2838,12 @@ AddScheduleManagerCmd::AddScheduleManagerCmd( Project &node, ScheduleManager *sm
 {
 }
 
-AddScheduleManagerCmd::AddScheduleManagerCmd( ScheduleManager *parent, ScheduleManager *sm, const QString& name )
+AddScheduleManagerCmd::AddScheduleManagerCmd( ScheduleManager *parent, ScheduleManager *sm, int index, const QString& name )
     : NamedCommand( name ),
     m_node( parent->project() ),
     m_parent( parent ),
     m_sm( sm ),
-    m_index( -1 ),
+    m_index( index ),
     m_exp( sm->expected() ),
     m_opt( sm->optimistic() ),
     m_pess( sm->pessimistic() ),
@@ -2799,7 +2861,7 @@ AddScheduleManagerCmd::~AddScheduleManagerCmd()
 
 void AddScheduleManagerCmd::execute()
 {
-    m_node.addScheduleManager( m_sm, m_parent );
+    m_node.addScheduleManager( m_sm, m_parent, m_index );
     m_sm->setExpected( m_exp );
     m_sm->setOptimistic( m_opt );
     m_sm->setPessimistic( m_pess );
@@ -2808,7 +2870,7 @@ void AddScheduleManagerCmd::execute()
 
 void AddScheduleManagerCmd::unexecute()
 {
-    m_index = m_node.takeScheduleManager( m_sm );
+    m_node.takeScheduleManager( m_sm );
     m_sm->setExpected( 0 );
     m_sm->setOptimistic( 0 );
     m_sm->setPessimistic( 0 );
@@ -2816,9 +2878,10 @@ void AddScheduleManagerCmd::unexecute()
 }
 
 DeleteScheduleManagerCmd::DeleteScheduleManagerCmd( Project &node, ScheduleManager *sm, const QString& name )
-    : AddScheduleManagerCmd( node, sm, name )
+    : AddScheduleManagerCmd( node, sm, -1, name )
 {
     m_mine = false;
+    m_index = m_parent ? m_parent->indexOf( sm ) : node.indexOf( sm );
     foreach ( ScheduleManager *s, sm->children() ) {
         cmd.addCommand( new DeleteScheduleManagerCmd( node, s ) );
     }
@@ -2960,22 +3023,20 @@ CalculateScheduleCmd::CalculateScheduleCmd( Project &node, ScheduleManager *sm,
 
 void CalculateScheduleCmd::execute()
 {
+    Q_ASSERT( m_sm );
     if ( m_first ) {
-        QApplication::setOverrideCursor( Qt::WaitCursor );
         m_sm->calculateSchedule();
-        Q_ASSERT( m_sm );
         if ( m_sm->calculationResult() != ScheduleManager::CalculationCanceled ) {
             m_first = false;
-            m_newexpected = m_sm->expected();
-            m_newoptimistic = m_sm->optimistic();
-            m_newpessimistic = m_sm->pessimistic();
         }
-        QApplication::restoreOverrideCursor();
-        return;
+        m_newexpected = m_sm->expected();
+        m_newoptimistic = m_sm->optimistic();
+        m_newpessimistic = m_sm->pessimistic();
+    } else {
+        m_sm->setExpected( m_newexpected );
+        m_sm->setOptimistic( m_newoptimistic );
+        m_sm->setPessimistic( m_newpessimistic );
     }
-    m_sm->setExpected( m_newexpected );
-    m_sm->setOptimistic( m_newoptimistic );
-    m_sm->setPessimistic( m_newpessimistic );
 }
 
 void CalculateScheduleCmd::unexecute()
@@ -2983,7 +3044,7 @@ void CalculateScheduleCmd::unexecute()
     if ( m_sm->scheduling() ) {
         // terminate scheduling
         QApplication::setOverrideCursor( Qt::WaitCursor );
-        m_sm->stopCalculation();
+        m_sm->haltCalculation();
         m_first = true;
         QApplication::restoreOverrideCursor();
 
@@ -2991,7 +3052,6 @@ void CalculateScheduleCmd::unexecute()
     m_sm->setExpected( m_oldexpected );
     m_sm->setOptimistic( m_oldoptimistic );
     m_sm->setPessimistic( m_oldpessimistic );
-
 }
 
 //------------------------
@@ -3400,7 +3460,7 @@ void InsertProjectCmd::addCalendars( Calendar *calendar, Calendar *parent, QMap<
     }
     Calendar *cal = m_project->findCalendar( calendar->id() );
     if ( cal == 0 ) {
-        addCommand( new CalendarAddCmd( m_project, calendar, par, "Calendar" ) );
+        addCommand( new CalendarAddCmd( m_project, calendar, -1, par, "Calendar" ) );
     } else {
         map[ calendar ] = cal;
     }
