@@ -105,6 +105,8 @@ public:
     virtual int type() const = 0;
     
     QString typeToString( bool trans = false ) const;
+    static QString typeToString( NodeTypes typ, bool trans = false );
+    static QStringList typeToStringList( bool trans );
     /**
      * Returns a pointer to the project node (main- or sub-project)
      * Returns 0 if no project exists.
@@ -264,6 +266,8 @@ public:
     /// within the duration of this node
     virtual void calcResourceOverbooked();
 
+    /// Return the scheduling status of schedule @p id. If @p trans is true, text is translated
+    QStringList schedulingStatus( long id, bool trans = false ) const;
     /// EstimateType == Estimate, but no resource is requested
     bool resourceError( long id = CURRENTSCHEDULE ) const;
     /// The assigned resource is overbooked
@@ -279,15 +283,15 @@ public:
     /// The assigned resources can not fullfil the estimated effort.
     virtual bool effortMetError( long /*id*/ = CURRENTSCHEDULE ) const { return false; }
     
-    virtual EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const=0;
-    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const=0;
+    virtual EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const=0;
+    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const=0;
 
     /// Returns the total planned effort for this task (or subtasks) 
-    virtual Duration plannedEffort( long id = CURRENTSCHEDULE ) const { Q_UNUSED(id); return Duration::zeroDuration; }
+    virtual Duration plannedEffort( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const { Q_UNUSED(id); return Duration::zeroDuration; }
     /// Returns the total planned effort for this task (or subtasks) on date
-    virtual Duration plannedEffort(const QDate &, long id = CURRENTSCHEDULE ) const { Q_UNUSED(id); return Duration::zeroDuration; }
+    virtual Duration plannedEffort(const QDate &, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const { Q_UNUSED(id); return Duration::zeroDuration; }
     /// Returns the planned effort up to and including date
-    virtual Duration plannedEffortTo(const QDate &, long id = CURRENTSCHEDULE ) const { Q_UNUSED(id); return Duration::zeroDuration; }
+    virtual Duration plannedEffortTo(const QDate &, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const { Q_UNUSED(id); return Duration::zeroDuration; }
     
     /// Returns the total actual effort for this task (or subtasks) 
     virtual Duration actualEffort() const { return Duration::zeroDuration; }
@@ -295,23 +299,23 @@ public:
     virtual Duration actualEffort(const QDate &/*date*/ ) const { return Duration::zeroDuration; }
     /// Returns the total actual effort for this task (or subtasks) up to and including date
     virtual Duration actualEffortTo(const QDate &/*date*/ ) const { return Duration::zeroDuration; }
-    virtual EffortCostMap actualEffortCostPrDay(const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const=0;
+    virtual EffortCostMap actualEffortCostPrDay(const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const=0;
     /// Returns the actual effort and cost pr day used by @p resource
-    virtual EffortCostMap actualEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const=0;
+    virtual EffortCostMap actualEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const=0;
     
     /**
      * Planned cost is the sum total of all resources and other costs
      * planned for this node.
      */
-    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE ) const;
+    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     
     /// Planned cost on date
-    virtual double plannedCost(const QDate &/*date*/, long id = CURRENTSCHEDULE ) const { Q_UNUSED(id); return 0; }
+    virtual double plannedCost(const QDate &/*date*/, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const { Q_UNUSED(id); return 0; }
     /**
      * Planned cost from start of activity up to and including date
      * is the sum of all resource costs and other costs planned for this node.
      */
-    virtual double plannedCostTo(const QDate &/*date*/, long id = CURRENTSCHEDULE ) const { Q_UNUSED(id); return 0; }
+    virtual double plannedCostTo(const QDate &/*date*/, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const { Q_UNUSED(id); return 0; }
     /**
      * Actual cost is the sum total of the reported costs actually used
      * for this node.
@@ -692,6 +696,8 @@ public:
     void setType(const QString& type);
     /// Return estimate type as a string. If @p trans is true, it's translated
     QString typeToString( bool trans=false ) const;
+    /// Return estimate type @p typ as a string. If @p trans is true, it's translated
+    static QString typeToString( Estimate::Type typ, bool trans=false );
     /// Return a stringlist of all estimate types. Translated if @p trans = true.
     static QStringList typeToStringList( bool trans=false );
     
@@ -789,12 +795,14 @@ public:
     Duration pertPessimistic() const;
     
     /// Convert the duration @p value (in milliseconds) to a value in @p unit, using the scaling factors in @p scales
-    static double scale( const Duration &value, Duration::Unit unit, const QList<double> &scales );
+    static double scale( const Duration &value, Duration::Unit unit, const QList<qint64> &scales );
     /// Convert the duration @p value (in @p unit) to a value in milliseconds (base unit), using the scaling factors in @p scales
-    static Duration scale( double value, Duration::Unit unit, const QList<double> &scales );
+    static Duration scale( double value, Duration::Unit unit, const QList<qint64> &scales );
 
+    /// Return a list of default scales scaling factors
+    static QList<qint64> defaultScales();
     /// Return a list of scaling factors fetched from the projects standard worktime
-    QList<double> scales() const;
+    QList<qint64> scales() const;
     
     /// Load from xml document
     bool load(KoXmlElement &element, XMLLoaderObject &status);
