@@ -230,6 +230,7 @@ public:
 protected:
     void copy( const Completion &copy);
     double averageCostPrHour( const QDate &date, long id ) const;
+    QPair<QDate, QDate> actualStartEndDates() const;
 
 private:
     Node *m_node;
@@ -287,7 +288,7 @@ public:
     /// Return schedule manager
     ScheduleManager *scheduleManager() const { return m_manager; }
     /// Return the schedule id, or NOTSCHEDULED if no schedule manager is set
-    long id() const { return m_manager ? m_manager->id() : NOTSCHEDULED; }
+    long id() const { return m_manager ? m_manager->scheduleId() : NOTSCHEDULED; }
 
     Completion &completion();
     const Completion &completion() const;
@@ -377,8 +378,6 @@ public:
     void addRequest(ResourceGroup *group, int numResources);
     void addRequest(ResourceGroupRequest *request);
     void takeRequest(ResourceGroupRequest *request);
-//    int units() const;
-    int workUnits() const;
     void makeAppointments();
     virtual QStringList requestNameList() const;
     virtual QList<Resource*> requestedResources() const;
@@ -410,19 +409,19 @@ public:
      * Returns a list of planned effort and cost for this task
      * for the interval start, end inclusive
      */
-    virtual EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
+    virtual EffortCostMap plannedEffortCostPrDay(const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /**
      * Returns a list of planned effort and cost for the @p resource
      * for the interval @p start, @p end inclusive, useing schedule with identity @p id
      */
-    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
+    virtual EffortCostMap plannedEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     
     /// Returns the total planned effort for this task (or subtasks) 
-    virtual Duration plannedEffort( long id = CURRENTSCHEDULE ) const;
+    virtual Duration plannedEffort( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Returns the total planned effort for this task (or subtasks) on date
-    virtual Duration plannedEffort(const QDate &date, long id = CURRENTSCHEDULE ) const;
+    virtual Duration plannedEffort(const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Returns the planned effort up to and including date
-    virtual Duration plannedEffortTo(const QDate &date, long id = CURRENTSCHEDULE ) const;
+    virtual Duration plannedEffortTo(const QDate &date, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     
     /// Returns the total actual effort for this task (or subtasks) 
     virtual Duration actualEffort() const;
@@ -434,28 +433,28 @@ public:
     /**
      * Returns the total planned cost for this task (or subtasks)
      */
-    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE ) const;
+    virtual EffortCost plannedCost( long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Planned cost on date
-    virtual double plannedCost(const QDate &/*date*/, long id = CURRENTSCHEDULE ) const;
+    virtual double plannedCost(const QDate &/*date*/, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Planned cost up to and including date
-    virtual double plannedCostTo(const QDate &/*date*/, long id = CURRENTSCHEDULE ) const;
+    virtual double plannedCostTo(const QDate &/*date*/, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     
     /**
      * Returns the actaually reported cost for this task (or subtasks)
      */
     virtual double actualCost() const;
     /// Actual cost on @p date
-    virtual double actualCost(const QDate &date) const;
+    virtual double actualCost( const QDate &date ) const;
     /// Returns actual effort and cost up to and including @p date
-    virtual EffortCost actualCostTo(const QDate &date) const;
+    virtual EffortCost actualCostTo( const QDate &date ) const;
 
     /**
      * Returns a list of actual effort and cost for this task
      * for the interval start, end inclusive
      */
-    virtual EffortCostMap actualEffortCostPrDay(const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE ) const;
+    virtual EffortCostMap actualEffortCostPrDay( const QDate &start, const QDate &end,  long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
     /// Returns the actual effort and cost pr day used by @p resource
-    virtual EffortCostMap actualEffortCostPrDay(const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE ) const;
+    virtual EffortCostMap actualEffortCostPrDay( const Resource *resource, const QDate &start, const QDate &end, long id = CURRENTSCHEDULE, EffortCostCalculationType = ECCT_All ) const;
 
     /// Returns the effort planned to be used to reach the actual percent finished
     virtual Duration budgetedWorkPerformed( const QDate &date, long id = CURRENTSCHEDULE ) const;
@@ -562,6 +561,11 @@ public:
      */
     virtual uint state( long id = CURRENTSCHEDULE ) const;
 
+    /// Check if this node has any dependent child nodes
+    virtual bool isEndNode() const;
+    /// Check if this node has any dependent parent nodes
+    virtual bool isStartNode() const;
+    
     QList<Relation*> parentProxyRelations() const { return  m_parentProxyRelations; }
     QList<Relation*> childProxyRelations() const { return  m_childProxyRelations; }
 
@@ -594,12 +598,7 @@ signals:
     void workPackageToBeRemoved( Node *node, int row );
     void workPackageRemoved( Node *node );
 
-protected:
-    /// Check if this node has any dependent child nodes
-    virtual bool isEndNode() const;
-    /// Check if this node has any dependent parent nodes
-    virtual bool isStartNode() const;
-    
+public:
     virtual void initiateCalculation(MainSchedule &sch);
     /**
      * Sets up the lists used for calculation.
