@@ -45,6 +45,8 @@
 #include <QTextTableCell>
 #include <QLineEdit>
 #include <QItemSelection>
+#include <QApplication>
+#include <QResizeEvent>
 
 #include <kicon.h>
 #include <kaction.h>
@@ -58,11 +60,17 @@
 #include <kaccelgen.h>
 #include <kactioncollection.h>
 
-#include "plotting/kplotwidget.h"
-#include "plotting/kplotobject.h"
-#include "plotting/kplotaxis.h"
+#include "KDChartChart"
+#include "KDChartAbstractCoordinatePlane"
+#include "KDChartBarDiagram"
+#include "KDChartLineDiagram"
+#include "KDChartCartesianAxis"
+#include "KDChartCartesianCoordinatePlane"
+#include "KDChartLegend"
+#include "KDChartBackgroundAttributes"
+#include "KDChartGridAttributes"
 
-#include <kplotpoint.h>
+using namespace KDChart;
 
 namespace KPlato
 {
@@ -77,9 +85,9 @@ TaskStatusTreeView::TaskStatusTreeView( QWidget *parent )
     //setSelectionBehavior( QAbstractItemView::SelectItems );
     setSelectionMode( QAbstractItemView::ExtendedSelection );
     setStretchLastSection( false );
-    
+
     createItemDelegates( m );
-    
+
     QList<int> lst1; lst1 << 1 << -1; // only display column 0 (NodeName) in left view
     masterView()->setDefaultColumns( QList<int>() << 0 );
     QList<int> show;
@@ -94,7 +102,7 @@ TaskStatusTreeView::TaskStatusTreeView( QWidget *parent )
             << NodeModel::NodeActualFinish
             << NodeModel::NodeStatusNote;
 
-    QList<int> lst2; 
+    QList<int> lst2;
     for ( int i = 0; i < m->columnCount(); ++i ) {
         if ( ! show.contains( i ) ) {
             lst2 << i;
@@ -157,7 +165,7 @@ void TaskStatusTreeView::setProject( Project *project )
     model()->setProject( project );
 }
 
-void TaskStatusTreeView::dragMoveEvent(QDragMoveEvent *event)
+void TaskStatusTreeView::dragMoveEvent(QDragMoveEvent */*event*/)
 {
 /*    if (dragDropMode() == InternalMove
         && (event->source() != this || !(event->possibleActions() & Qt::MoveAction)))
@@ -214,7 +222,7 @@ TaskStatusView::TaskStatusView( KoDocument *part, QWidget *parent )
     connect( model(), SIGNAL( executeCommand( QUndoCommand* ) ), part, SLOT( addCommand( QUndoCommand* ) ) );
 
     connect( m_view, SIGNAL( contextMenuRequested( const QModelIndex&, const QPoint& ) ), SLOT( slotContextMenuRequested( const QModelIndex&, const QPoint& ) ) );
-    
+
     connect( m_view, SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
 }
 
@@ -229,7 +237,7 @@ void TaskStatusView::setScheduleManager( ScheduleManager *sm )
     static_cast<TaskStatusItemModel*>( m_view->model() )->setScheduleManager( sm );
 }
 
-Node *TaskStatusView::currentNode() const 
+Node *TaskStatusView::currentNode() const
 {
     return m_view->model()->node( m_view->selectionModel()->currentIndex() );
 }
@@ -297,7 +305,7 @@ void TaskStatusView::setupGui()
     // Add the context menu actions for the view options
     connect(m_view->actionSplitView(), SIGNAL(triggered(bool) ), SLOT(slotSplitView()));
     addContextAction( m_view->actionSplitView() );
-    
+
     createOptionAction();
 }
 
@@ -327,9 +335,9 @@ bool TaskStatusView::loadContext( const KoXmlElement &context )
 {
     kDebug();
     m_view->setPeriod( context.attribute( "period", QString("%1").arg( m_view->defaultPeriod() ) ).toInt() );
-    
+
     m_view->setPeriodType( context.attribute( "periodtype", QString("%1").arg( m_view->defaultPeriodType() ) ).toInt() );
-    
+
     m_view->setWeekday( context.attribute( "weekday", QString("%1").arg( m_view->defaultWeekday() ) ).toInt() );
     return m_view->loadContext( model()->columnMap(), context );
 }
@@ -354,7 +362,7 @@ TaskStatusViewSettingsPanel::TaskStatusViewSettingsPanel( TaskStatusTreeView *vi
     m_view( view )
 {
     setupUi( this );
-    
+
     QStringList lst;
     for ( int i = 1; i <= 7; ++i ) {
         lst << KGlobal::locale()->calendar()->weekDayName( i );
@@ -367,7 +375,7 @@ TaskStatusViewSettingsPanel::TaskStatusViewSettingsPanel( TaskStatusTreeView *vi
         default: break;
     }
     weekdays->setCurrentIndex( m_view->weekday() - 1 );
-    
+
     connect( period, SIGNAL( valueChanged( int ) ), SIGNAL( changed() ) );
     connect( useWeekday, SIGNAL( toggled( bool ) ), SIGNAL( changed() ) );
     connect( useCurrentDate, SIGNAL( toggled( bool ) ), SIGNAL( changed() ) );
@@ -407,7 +415,7 @@ TaskStatusViewSettingsDialog::TaskStatusViewSettingsDialog( TaskStatusTreeView *
     KPageWidgetItem *page = insertWidget( 0, panel, i18n( "General" ), i18n( "General Settings" ) );
     setCurrentPage( page );
     //connect( panel, SIGNAL( changed( bool ) ), this, SLOT( enableButtonOk( bool ) ) );
-    
+
     connect( this, SIGNAL( okClicked() ), panel, SLOT( slotOk() ) );
     connect( this, SIGNAL( defaultClicked() ), panel, SLOT( setDefault() ) );
 }
@@ -424,7 +432,7 @@ ProjectStatusView::ProjectStatusView( KoDocument *part, QWidget *parent )
     l->addWidget( m_view );
 
     setupGui();
-    
+
     connect( m_view, SIGNAL( customContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
 
 
@@ -447,11 +455,6 @@ void ProjectStatusView::setProject( Project *project )
     m_view->setProject( project );
 }
 
-void ProjectStatusView::draw()
-{
-    m_view->draw();
-}
-
 void ProjectStatusView::setGuiActive( bool activate )
 {
     kDebug()<<activate;
@@ -497,16 +500,302 @@ PerformanceStatusBase::PerformanceStatusBase( QWidget *parent )
     labelBCWS->setToolTip( ToolTip::nodeBCWS() );
     labelBCWP->setToolTip( ToolTip::nodeBCWP() );
     labelACWP->setToolTip( ToolTip::nodeACWP() );
-    labelPI->setToolTip(  i18nc( "@info:tooltip", "Performance indexes" ) );
+
     labelCPI->setToolTip( i18nc( "@info:tooltip", "Cost performance index (BCWP/ACWP)" ) );
     labelSPI->setToolTip( ToolTip::nodePerformanceIndex() );
 
-    plotwidget->setAntialiasing(false);
-    plotwidget->setShowGrid(false);
-    connect( &m_model, SIGNAL( reset() ), SLOT( slotReset() ) );
-    
-    setContextMenuPolicy ( Qt::DefaultContextMenu );
+    BackgroundAttributes backgroundAttrs( ui_chart->backgroundAttributes() );
+    backgroundAttrs.setVisible( true );
+    backgroundAttrs.setBrush( Qt::white );
+    ui_chart->setBackgroundAttributes( backgroundAttrs );
 
+    m_legend = new Legend( ui_chart );
+    ui_chart->replaceLegend( m_legend );
+    m_legend->setObjectName( "Chart legend" );
+
+    backgroundAttrs = m_legend->backgroundAttributes();
+    m_legend->setBackgroundAttributes( backgroundAttrs );
+    backgroundAttrs.setVisible( true );
+    backgroundAttrs.setBrush( Qt::white );
+
+    m_legend->setPosition( Position::East );
+    //m_legend->setAlignment( (Qt::Alignment)(Qt::AlignTop | Qt::AlignCenter) );
+    m_legenddiagram.setModel( &m_chartmodel );
+    m_legenddiagram.setObjectName( "Legend diagram" );
+    m_legend->setDiagram( &m_legenddiagram );
+
+    // get rid of the default coordinate plane
+    AbstractCoordinatePlane *p = ui_chart->coordinatePlane();
+    ui_chart->takeCoordinatePlane( p );
+    delete p;
+
+    createBarChart();
+    createLineChart();
+    setupChart();
+
+    connect(&m_chartmodel, SIGNAL(modelReset()), SLOT(slotUpdate()));
+    setContextMenuPolicy ( Qt::DefaultContextMenu );
+}
+
+void PerformanceStatusBase::setChartInfo( const PerformanceChartInfo &info )
+{
+    if ( info != m_chartinfo ) {
+        m_chartinfo = info;
+        setupChart();
+    }
+}
+
+void PerformanceStatusBase::refreshChart()
+{
+    // NOTE: Force grid/axis recalculation, couldn't find a better way :(
+    QResizeEvent event( ui_chart->size(), QSize() );
+    QApplication::sendEvent( ui_chart, &event );
+}
+
+void PerformanceStatusBase::createBarChart()
+{
+    m_barchart.effortplane = new CartesianCoordinatePlane( ui_chart );
+    m_barchart.effortplane->setObjectName( "Bar chart, Effort" );
+    m_barchart.costplane = new CartesianCoordinatePlane( ui_chart );
+    m_barchart.costplane->setObjectName( "Bar chart, Cost" );
+
+    BarDiagram *effortdiagram = new BarDiagram( ui_chart, m_barchart.effortplane );
+    effortdiagram->setObjectName( "Effort diagram" );
+
+    m_barchart.dateaxis = new CartesianAxis();
+    m_barchart.dateaxis->setPosition( CartesianAxis::Bottom );
+
+    m_barchart.effortaxis = new CartesianAxis( effortdiagram );
+    m_barchart.effortaxis->setPosition( CartesianAxis::Right );
+    effortdiagram->addAxis( m_barchart.effortaxis );
+    m_barchart.effortplane->addDiagram( effortdiagram );
+
+    // Hide cost in effort diagram
+    effortdiagram->setHidden( 0, true );
+    effortdiagram->setHidden( 1, true );
+    effortdiagram->setHidden( 2, true );
+    m_barchart.effortproxy.setZeroColumns( QList<int>() << 0 << 1 << 2 );
+
+    m_barchart.effortproxy.setSourceModel( &m_chartmodel );
+    effortdiagram->setModel( &(m_barchart.effortproxy) );
+
+    BarDiagram *costdiagram = new BarDiagram( ui_chart, m_barchart.costplane );
+    costdiagram->setObjectName( "Cost diagram" );
+
+    m_barchart.costaxis = new CartesianAxis( costdiagram );
+    m_barchart.costaxis->setPosition( CartesianAxis::Left );
+    costdiagram->addAxis( m_barchart.costaxis );
+    m_barchart.costplane->addDiagram( costdiagram );
+
+    // Hide effort in cost diagram
+    costdiagram->setHidden( 3, true );
+    costdiagram->setHidden( 4, true );
+    costdiagram->setHidden( 5, true );
+    m_barchart.costproxy.setZeroColumns( QList<int>() << 3 << 4 << 5 );
+
+    m_barchart.costproxy.setObjectName( "Bar: Cost" );
+    m_barchart.costproxy.setSourceModel( &m_chartmodel );
+    costdiagram->setModel( &(m_barchart.costproxy) );
+}
+
+void PerformanceStatusBase::createLineChart()
+{
+    m_linechart.effortplane = new CartesianCoordinatePlane( ui_chart );
+    m_linechart.effortplane->setObjectName( "Line chart, Effort" );
+    m_linechart.effortplane->setRubberBandZoomingEnabled( true );
+    m_linechart.costplane = new CartesianCoordinatePlane( ui_chart );
+    m_linechart.costplane->setObjectName( "Line chart, Cost" );
+    m_linechart.costplane->setRubberBandZoomingEnabled( true );
+
+    LineDiagram *effortdiagram = new LineDiagram( ui_chart, m_linechart.effortplane );
+    effortdiagram->setObjectName( "Effort diagram" );
+
+    m_linechart.dateaxis = new CartesianAxis();
+    m_linechart.dateaxis->setPosition( CartesianAxis::Bottom );
+
+    m_linechart.effortaxis = new CartesianAxis( effortdiagram );
+    m_linechart.effortaxis->setPosition( CartesianAxis::Right );
+    effortdiagram->addAxis( m_linechart.effortaxis );
+    m_linechart.effortplane->addDiagram( effortdiagram );
+
+    // Hide cost in effort diagram
+    effortdiagram->setHidden( 0, true );
+    effortdiagram->setHidden( 1, true );
+    effortdiagram->setHidden( 2, true );
+    m_linechart.effortproxy.setZeroColumns( QList<int>() << 0 << 1 << 2 );
+
+    m_linechart.effortproxy.setObjectName( "Line: Effort" );
+    m_linechart.effortproxy.setSourceModel( &m_chartmodel );
+    effortdiagram->setModel( &(m_linechart.effortproxy) );
+
+    LineDiagram *costdiagram = new LineDiagram( ui_chart, m_linechart.costplane );
+    costdiagram->setObjectName( "Cost diagram" );
+
+    m_linechart.costaxis = new CartesianAxis( costdiagram );
+    m_linechart.costaxis->setPosition( CartesianAxis::Left );
+    costdiagram->addAxis( m_linechart.costaxis );
+    m_linechart.costplane->addDiagram( costdiagram );
+    // Hide effort in cost diagram
+    costdiagram->setHidden( 3, true );
+    costdiagram->setHidden( 4, true );
+    costdiagram->setHidden( 5, true );
+
+    m_linechart.costproxy.setObjectName( "Line: Cost" );
+    m_linechart.costproxy.setZeroColumns( QList<int>() << 3 << 4 << 5 );
+    m_linechart.costproxy.setSourceModel( &m_chartmodel );
+    costdiagram->setModel( &(m_linechart.costproxy) );
+
+
+}
+
+void PerformanceStatusBase::setupChart()
+{
+    while ( ! ui_chart->coordinatePlanes().isEmpty() ) {
+        ui_chart->takeCoordinatePlane( ui_chart->coordinatePlanes().last() );
+    }
+    if ( m_chartinfo.showBarChart ) {
+        setupChart( m_barchart );
+    } else if ( m_chartinfo.showLineChart ) {
+        setupChart( m_linechart );
+    } else {
+        setupChart( m_linechart );
+    }
+    kDebug()<<"Planes:"<<ui_chart->coordinatePlanes();
+    foreach ( AbstractCoordinatePlane *pl, ui_chart->coordinatePlanes() ) {
+        CartesianCoordinatePlane *p = dynamic_cast<CartesianCoordinatePlane*>( pl );
+        if ( p == 0 ) continue;
+        GridAttributes ga = p->globalGridAttributes();
+        ga.setGridVisible( p->referenceCoordinatePlane() == 0 );
+        p->setGlobalGridAttributes( ga );
+    }
+    m_legend->setDatasetHidden( 0, ! ( m_chartinfo.showCost && m_chartinfo.showBCWSCost ) );
+    m_legend->setDatasetHidden( 1, ! ( m_chartinfo.showCost && m_chartinfo.showBCWPCost ) );
+    m_legend->setDatasetHidden( 2, ! ( m_chartinfo.showCost && m_chartinfo.showACWPCost ) );
+    m_legend->setDatasetHidden( 3, ! ( m_chartinfo.showEffort && m_chartinfo.showBCWSEffort ) );
+    m_legend->setDatasetHidden( 4, ! ( m_chartinfo.showEffort && m_chartinfo.showBCWPEffort ) );
+    m_legend->setDatasetHidden( 5, ! ( m_chartinfo.showEffort && m_chartinfo.showACWPEffort ) );
+    
+    setEffortValuesVisible( m_chartinfo.showEffort );
+    setCostValuesVisible( m_chartinfo.showCost );
+    refreshChart();
+}
+
+void PerformanceStatusBase::setEffortValuesVisible( bool visible )
+{
+    labelEffort->setVisible( visible );
+    bcwsEffort->setVisible( visible );
+    bcwpEffort->setVisible( visible );
+    acwpEffort->setVisible( visible );
+    spiEffort->setVisible( visible );
+    cpiEffort->setVisible( visible );
+}
+
+void PerformanceStatusBase::setCostValuesVisible( bool visible )
+{
+    labelCost->setVisible( visible );
+    bcwsCost->setVisible( visible );
+    bcwpCost->setVisible( visible );
+    acwpCost->setVisible( visible );
+    spiCost->setVisible( visible );
+    cpiCost->setVisible( visible );
+}
+
+void PerformanceStatusBase::setupChart( ChartContents &cc )
+{
+    QList<int> erc, ezc, crc, czc; // sourcemodel column numbers
+    int effort_start_column = 3; // proxy column number
+
+    const PerformanceChartInfo &info = m_chartinfo;
+    kDebug()<<"cost="<<info.showCost<<"effort="<<info.showEffort;
+    static_cast<AbstractCartesianDiagram*>( cc.effortplane->diagram() )->takeAxis( cc.dateaxis );
+    static_cast<AbstractCartesianDiagram*>( cc.costplane->diagram() )->takeAxis( cc.dateaxis );
+    cc.costplane->setReferenceCoordinatePlane( 0 );
+    if ( info.showEffort ) {
+        // filter cost columns if cost is *not* shown, else hide them and zero out
+        if ( ! info.showCost ) {
+            erc << 0 << 1 << 2;
+            effort_start_column = 0;
+        } else {
+            ezc << 0 << 1 << 2;
+            cc.effortplane->diagram()->setHidden( 0, true );
+            cc.effortplane->diagram()->setHidden( 1, true );
+            cc.effortplane->diagram()->setHidden( 2, true );
+        }
+        // if cost is shown don't return a cost value or else it goes into the effort axis scale calculation
+        //cc.effortproxy.setZeroColumns( info.showCost ? QList<int>() << 0 << 1 << 2 : QList<int>() << 3 << 4 << 5  );
+        cc.effortaxis->setPosition( info.showCost ? CartesianAxis::Right : CartesianAxis::Left );
+        ui_chart->addCoordinatePlane( cc.effortplane );
+
+        static_cast<AbstractCartesianDiagram*>( cc.effortplane->diagram() )->addAxis( cc.dateaxis );
+        cc.effortplane->setGridNeedsRecalculate();
+    }
+    if ( info.showCost ) {
+        // Should never get any effort values in cost diagram
+        czc << 3 << 4 << 5;
+        // remove effort columns from cost if no effort is shown, else hide them
+        if ( ! info.showEffort ) {
+            crc << 3 << 4 << 5;
+        } else {
+            cc.costplane->diagram()->setHidden( 3, true );
+            cc.costplane->diagram()->setHidden( 4, true );
+            cc.costplane->diagram()->setHidden( 5, true );
+        }
+
+        cc.costplane->setReferenceCoordinatePlane( info.showEffort ? cc.effortplane : 0 );
+        ui_chart->addCoordinatePlane( cc.costplane );
+
+        static_cast<AbstractCartesianDiagram*>( cc.costplane->diagram() )->addAxis( cc.dateaxis );
+        cc.costplane->setGridNeedsRecalculate();
+
+        cc.costplane->diagram()->setHidden( 0, ! info.showBCWSCost );
+        cc.costplane->diagram()->setHidden( 1, ! info.showBCWPCost );
+        cc.costplane->diagram()->setHidden( 2, ! info.showACWPCost );
+    }
+    
+    if ( info.showEffort ) {
+        cc.effortplane->diagram()->setHidden( effort_start_column, ! info.showBCWSEffort );
+        cc.effortplane->diagram()->setHidden( effort_start_column+1, ! info.showBCWPEffort );
+        cc.effortplane->diagram()->setHidden( effort_start_column+2, ! info.showACWPEffort );
+        cc.effortaxis->setCachedSizeDirty();
+        cc.effortproxy.reset();
+        cc.effortproxy.setZeroColumns( ezc );
+        cc.effortproxy.setRejectColumns( erc );
+    }
+    if ( info.showCost ) {
+        cc.costaxis->setCachedSizeDirty();
+        cc.costproxy.reset();
+        cc.costproxy.setZeroColumns( czc );
+        cc.costproxy.setRejectColumns( crc );
+    }
+#if 1
+    kDebug()<<"Effort:"<<info.showEffort;
+    if ( info.showEffort && cc.effortproxy.rowCount() > 0 ) {
+        kDebug()<<"Effort:"<<info.showEffort<<"columns ="<<cc.effortproxy.columnCount()
+            <<"reject="<<cc.effortproxy.rejectColumns()
+            <<"zero="<<cc.effortproxy.zeroColumns();
+        int row = cc.effortproxy.rowCount()-1;
+        for ( int i = 0; i < cc.effortproxy.columnCount(); ++i ) {
+            kDebug()<<"data ("<<row<<","<<i<<":"<<cc.effortproxy.index(row,i).data().toString()<<(cc.effortplane->diagram()->isHidden(i)?"hide":"show");;
+        }
+    }
+    kDebug()<<"Cost:"<<info.showCost;
+    if ( info.showCost && cc.costproxy.rowCount() > 0 ) {
+        kDebug()<<"Cost:"<<info.showCost<<"columns ="<<cc.costproxy.columnCount()
+            <<"reject="<<cc.costproxy.rejectColumns()
+            <<"zero="<<cc.costproxy.zeroColumns();
+        int row = cc.costproxy.rowCount()-1;
+        for ( int i = 0; i < cc.costproxy.columnCount(); ++i ) {
+            kDebug()<<"data ("<<row<<","<<i<<":"<<cc.costproxy.index(row,i).data().toString()<<(cc.costplane->diagram()->isHidden(i)?"hide":"show");;
+        }
+    }
+
+    foreach( AbstractCoordinatePlane *p, ui_chart->coordinatePlanes() ) {
+        kDebug()<<p<<"refrences:"<<p->referenceCoordinatePlane();
+        foreach ( AbstractDiagram *d, p->diagrams() ) {
+            qDebug()<<p<<"diagram:"<<d;
+        }
+    }
+#endif
 }
 
 void PerformanceStatusBase::contextMenuEvent( QContextMenuEvent *event )
@@ -515,44 +804,49 @@ void PerformanceStatusBase::contextMenuEvent( QContextMenuEvent *event )
     emit customContextMenuRequested( event->globalPos() );
 }
 
-void PerformanceStatusBase::slotReset()
-{
-    //kDebug();
-    draw();
-}
-
 void PerformanceStatusBase::slotUpdate()
 {
     //kDebug();
     drawValues();
+    refreshChart();
 }
 
 void PerformanceStatusBase::setScheduleManager( ScheduleManager *sm )
 {
     //kDebug();
     m_manager = sm;
-    m_model.setScheduleManager( sm );
+    m_chartmodel.setScheduleManager( sm );
 }
 
 void PerformanceStatusBase::setProject( Project *project )
 {
     if ( m_project ) {
-        disconnect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotUpdate() ) );
+        disconnect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLocaleChanged() ) );
     }
     m_project = project;
-    m_model.setProject( project );
     if ( m_project ) {
-        connect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotUpdate() ) );
+        connect( m_project, SIGNAL( localeChanged() ), this, SLOT( slotLocaleChanged() ) );
     }
+    m_chartmodel.setProject( project );
+
+    slotLocaleChanged();
 }
 
-void PerformanceStatusBase::draw()
+void PerformanceStatusBase::slotLocaleChanged()
 {
-    if ( m_project == 0 || m_manager == 0 ) {
+    kDebug();
+    KLocale *locale = m_project ? m_project->locale() : KGlobal::locale();
+    if ( locale ) {
+        m_linechart.costaxis->setTitleText( i18nc( "Chart axis title 1=currency symbol", "Cost (%1)", m_project->locale()->currencySymbol() ) );
+        m_linechart.effortaxis->setTitleText( i18nc( "Chart axis title", "Effort (hours)" ) );
+
+        m_barchart.costaxis->setTitleText( i18nc( "Chart axis title 1=currency symbol", "Cost (%1)", m_project->locale()->currencySymbol() ) );
+        m_barchart.effortaxis->setTitleText( i18nc( "Chart axis title", "Effort (hours)" ) );
+    }
+    if ( m_project == 0 ) {
         return;
     }
     drawValues();
-    drawPlot( *m_project, *m_manager );
 }
 
 void PerformanceStatusBase::drawValues()
@@ -561,192 +855,77 @@ void PerformanceStatusBase::drawValues()
         return;
     }
     KLocale *locale = m_project->locale();
-    const EffortCostMap &budget = m_model.bcwp();
-    const EffortCostMap &actual = m_model.acwp();
-    bcwsCost->setText( locale->formatMoney( budget.totalCost() ) );
+    QDate date = QDate::currentDate();
+
+    const EffortCostMap &budget = m_chartmodel.bcwp();
+    const EffortCostMap &actual = m_chartmodel.acwp();
+
+    double bc = budget.costTo( date );
+    bcwsCost->setText( locale->formatMoney( bc ) );
     bcwpCost->setText( locale->formatMoney( budget.bcwpTotalCost() ) );
     acwpCost->setText( locale->formatMoney( actual.totalCost() ) );
-    
+
+    double spi_ = 0.0;
+    if ( bc > 0.0 ) {
+        spi_ = budget.bcwpTotalCost() / bc;
+    }
     double cpi_ = 0.0;
     if ( actual.totalCost() > 0.0 ) {
         cpi_ = budget.bcwpTotalCost() / actual.totalCost();
     }
-    cpi->setText( locale->formatNumber( cpi_ ) );
-    
-    double bh = budget.totalEffort().toDouble( Duration::Unit_h);
+    spiCost->setText( locale->formatNumber( spi_ ) );
+    cpiCost->setText( locale->formatNumber( cpi_ ) );
+
+    double bh = budget.hoursTo( date );
     bcwsEffort->setText( locale->formatNumber( bh ) );
     bcwpEffort->setText( locale->formatNumber( budget.bcwpTotalEffort() ) );
     acwpEffort->setText( locale->formatNumber( actual.totalEffort().toDouble( Duration::Unit_h) ) );
 
-    double spi_ = 0.0;
+    spi_ = 0.0;
     if ( bh > 0.0 ) {
         spi_ = budget.bcwpTotalEffort() / bh;
     }
-    spi->setText( locale->formatNumber( spi_ ) );
+    cpi_ = 0.0;
+    if ( actual.totalCost() > 0.0 ) {
+        cpi_ = budget.bcwpTotalCost() / actual.totalCost();
+    }
+    spiEffort->setText( locale->formatNumber( spi_ ) );
+    cpiEffort->setText( locale->formatNumber( cpi_ ) );
 }
 
-void PerformanceStatusBase::drawPlot( Project &p, ScheduleManager &sm ) 
-{
-    //kDebug();
-    plotwidget->resetPlot();
-    int axisCount = m_model.axisCount();
-    for ( int i = 0; i < axisCount; ++i ) {
-        kDebug()<<i<<"of"<<axisCount;
-        ChartAxisIndex ai = m_model.axisIndex( i );
-        if ( ! ai.isValid() ) {
-            continue;
-        }
-        QList<QPointF> range = drawAxis( ai );
-        kDebug()<<i<<range;
-        if ( i == 0 ) {
-            plotwidget->setLimits( range[0].x(), range[0].y(), range[1].x(), range[1].y()  );
-            //kDebug()<<"Primary:"<<plotwidget->dataRect();
-        }
-        if ( i == 1 ) {
-            plotwidget->setSecondaryLimits( range[0].x(), range[0].y(), range[1].x(), range[1].y() );
-            //kDebug()<<"Secondary:"<<plotwidget->secondaryDataRect();
-        }
-        drawData( ai );
-    }
-   
-}
-
-QList<QPointF> PerformanceStatusBase::drawAxis( const ChartAxisIndex &idx ) 
-{
-    //kDebug();
-    int axisCount = m_model.axisCount( idx );
-    QList<QPointF> range;
-    for ( int i = 0; i < axisCount; ++i ) {
-        ChartAxisIndex ai = m_model.axisIndex( i, idx );
-        if ( ! ai.isValid() ) {
-            continue;
-        }
-        int type = m_model.axisData( ai, AbstractChartModel::AxisTypeRole ).toInt();
-        if ( type == AbstractChartModel::Axis_X ) {
-            //kDebug()<<"add x-axis";
-            range.prepend ( QPointF ( m_model.axisData( ai, AbstractChartModel::AxisMinRole ).toDouble(), m_model.axisData( ai, AbstractChartModel::AxisMaxRole ).toDouble() ) );
-            if ( idx.number() == 0 ) {
-                plotwidget->axis( KPlotWidget::BottomAxis )->setLabel( m_model.axisData( ai, Qt::DisplayRole ).toString() );
-                
-                plotwidget->axis( KPlotWidget::BottomAxis )->setStartDate( m_model.axisData( ai, AbstractChartModel::AxisStartDateRole ).toDate() );
-                plotwidget->axis( KPlotWidget::BottomAxis )->setTickLabelFormat( 'd' );
-            } else {
-                plotwidget->axis( KPlotWidget::TopAxis )->setLabel( m_model.axisData( ai, Qt::DisplayRole ).toString() );
-                
-                plotwidget->axis( KPlotWidget::TopAxis )->setStartDate( m_model.axisData( ai, AbstractChartModel::AxisStartDateRole ).toDate() );
-                plotwidget->axis( KPlotWidget::TopAxis )->setTickLabelFormat( 'w' );
-                
-                plotwidget->axis( KPlotWidget::TopAxis )->setTickLabelsShown( true );
-            }
-        } else if ( type == AbstractChartModel::Axis_Y ) {
-            //kDebug()<<"add y-axis"<<idx<<ai;
-            range.append ( QPointF ( m_model.axisData( ai, AbstractChartModel::AxisMinRole ).toDouble(), m_model.axisData( ai, AbstractChartModel::AxisMaxRole ).toDouble() ) );
-            if ( idx.number() == 0 ) {
-                QString s = m_model.axisData( ai, Qt::DisplayRole ).toString();
-                //kDebug()<<"LeftAxis:"<<s;
-                plotwidget->axis( KPlotWidget::LeftAxis )->setLabel( s );
-                
-                plotwidget->axis( KPlotWidget::LeftAxis )->setStartDate( m_model.axisData( ai, AbstractChartModel::AxisStartDateRole ).toDate() );
-            } else {
-                QString s = m_model.axisData( ai, Qt::DisplayRole ).toString();
-                //kDebug()<<"RightAxis:"<<s;
-                plotwidget->axis( KPlotWidget::RightAxis )->setLabel( s );
-            
-                plotwidget->axis( KPlotWidget::RightAxis )->setStartDate( m_model.axisData( ai, AbstractChartModel::AxisStartDateRole ).toDate() );
-            
-                plotwidget->axis( KPlotWidget::RightAxis )->setTickLabelsShown( true );
-            }
-        }
-        //kDebug()<<range;
-    }
-    return range;
-}
-
-void PerformanceStatusBase::drawData( const ChartAxisIndex &axisSet ) 
-{
-    //kDebug()<<axisSet;
-    int dataCount = m_model.dataSetCount( axisSet );
-    for ( int i = 0; i < dataCount; ++i ) {
-        ChartDataIndex di = m_model.index( i, axisSet );
-        if ( ! di.isValid() ) {
-            //kDebug()<<"Invalid index"<<di;
-            continue;
-        }
-        if ( m_model.hasChildren( di ) ) {
-            //kDebug()<<"sections";
-            int c = m_model.childCount( di );
-            for ( int ii = 0; ii < c; ++ii ) {
-                ChartDataIndex cidx = m_model.index( ii, di );
-                drawData( cidx, axisSet );
-            }
-        } else {
-            //kDebug()<<"no sections, go direct to data"<<di;
-            drawData( di, axisSet );
-        }
-    }
-}
-
-void PerformanceStatusBase::drawData( const ChartDataIndex &index, const ChartAxisIndex &axisSet )
-{
-    //kDebug()<<index;
-    QVariantList data;
-    int axisCount = m_model.axisCount( axisSet );
-    for ( int i = 0; i < axisCount; ++i ) {
-        ChartAxisIndex axis = m_model.axisIndex( i, axisSet );
-        //kDebug()<<"data axis"<<axis<<" set="<<axisSet;
-        data << m_model.data( index, axis );
-    }
-    //kDebug()<<data;
-    QVariantList xlst = data[0].toList();
-    QVariantList ylst = data[1].toList();
-    QVariant color = m_model.data( index, axisSet, Qt::ForegroundRole );
-    QVariant label = m_model.data( index, axisSet, AbstractChartModel::DataLabelRole );
-    KPlotObject *kpo = new KPlotObject( color.value<QColor>(), KPlotObject::Lines, 4 );
-    double factor = plotwidget->dataRect().height() / plotwidget->secondaryDataRect().height();
-    for (int i = 0; i < ylst.count(); ++i ) {
-        //kDebug()<<"Add point:"<<x[i].toInt() << y[i].toDouble();
-        double y = ylst[i].toDouble();
-        if ( axisSet.number() == 1 ) {
-            y *= factor;
-        }
-        KPlotPoint *p = new KPlotPoint( xlst[i].toInt(), y );
-        if ( label.isValid() && i == ylst.count() / 2 ) {
-            p->setLabel( label.toString() );
-        }
-        kpo->addPoint( p );
-    }
-    plotwidget->addPlotObject( kpo );
-}
 
 bool PerformanceStatusBase::loadContext( const KoXmlElement &context )
 {
     kDebug();
-    NodeChartModel::DataShown show;
-    show.showCost = context.attribute( "show-cost", "1" ).toInt();
-    show.showBCWSCost = context.attribute( "show-bcws-cost", "1" ).toInt();
-    show.showBCWPCost = context.attribute( "show-bcwp-cost", "1" ).toInt();
-    show.showACWPCost = context.attribute( "show-acwp-cost", "1" ).toInt();
-    
-    show.showEffort = context.attribute( "show-effort", "1" ).toInt();
-    show.showBCWSEffort = context.attribute( "show-bcws-effort", "1" ).toInt();
-    show.showBCWPEffort = context.attribute( "show-bcwp-effort", "1" ).toInt();
-    show.showACWPEffort = context.attribute( "show-acwp-effort", "1" ).toInt();
-    model()->setDataShown( show );
+    m_chartinfo.showBarChart = context.attribute( "show-bar-chart", "0" ).toInt();
+    m_chartinfo.showLineChart = context.attribute( "show-line-chart", "1" ).toInt();
+
+    m_chartinfo.showCost = context.attribute( "show-cost", "1" ).toInt();
+    m_chartinfo.showBCWSCost = context.attribute( "show-bcws-cost", "1" ).toInt();
+    m_chartinfo.showBCWPCost = context.attribute( "show-bcwp-cost", "1" ).toInt();
+    m_chartinfo.showACWPCost = context.attribute( "show-acwp-cost", "1" ).toInt();
+
+    m_chartinfo.showEffort = context.attribute( "show-effort", "1" ).toInt();
+    m_chartinfo.showBCWSEffort = context.attribute( "show-bcws-effort", "1" ).toInt();
+    m_chartinfo.showBCWPEffort = context.attribute( "show-bcwp-effort", "1" ).toInt();
+    m_chartinfo.showACWPEffort = context.attribute( "show-acwp-effort", "1" ).toInt();
+    kDebug()<<"Cost:"<<m_chartinfo.showCost<<"bcws="<<m_chartinfo.showBCWSCost<<"bcwp="<<m_chartinfo.showBCWPCost<<"acwp="<<m_chartinfo.showACWPCost;
+    kDebug()<<"Effort:"<<m_chartinfo.showCost<<"bcws="<<m_chartinfo.showBCWSCost<<"bcwp="<<m_chartinfo.showBCWPCost<<"acwp="<<m_chartinfo.showACWPCost;
+    setupChart();
     return true;
 }
 
 void PerformanceStatusBase::saveContext( QDomElement &context ) const
 {
-    NodeChartModel::DataShown show = model()->dataShown();
-    context.setAttribute( "show-cost", show.showCost );
-    context.setAttribute( "show-bcws-cost", show.showBCWSCost );
-    context.setAttribute( "show-bcwp-cost", show.showBCWPCost );
-    context.setAttribute( "show-acwp-cost", show.showACWPCost );
-    
-    context.setAttribute( "show-effort",  show.showEffort );
-    context.setAttribute( "show-bcws-effort", show.showBCWSEffort );
-    context.setAttribute( "show-bcwp-effort", show.showBCWPEffort );
-    context.setAttribute( "show-acwp-effort", show.showACWPEffort );
+    context.setAttribute( "show-cost", m_chartinfo.showCost );
+    context.setAttribute( "show-bcws-cost", m_chartinfo.showBCWSCost );
+    context.setAttribute( "show-bcwp-cost", m_chartinfo.showBCWPCost );
+    context.setAttribute( "show-acwp-cost", m_chartinfo.showACWPCost );
+
+    context.setAttribute( "show-effort",  m_chartinfo.showEffort );
+    context.setAttribute( "show-bcws-effort", m_chartinfo.showBCWSEffort );
+    context.setAttribute( "show-bcwp-effort", m_chartinfo.showBCWPEffort );
+    context.setAttribute( "show-acwp-effort", m_chartinfo.showACWPEffort );
 }
 
 //-----------------------------------
@@ -761,10 +940,10 @@ PerformanceStatusTreeView::PerformanceStatusTreeView( QWidget *parent )
     m_tree->setColumnsHidden( lst1 );
     m_tree->setSelectionMode( QAbstractItemView::ExtendedSelection );
     addWidget( m_tree );
-    
+
     m_chart = new PerformanceStatusBase( this );
     addWidget( m_chart );
-    
+
     connect( m_tree->selectionModel(), SIGNAL( selectionChanged( const QItemSelection &, const QItemSelection & ) ), SLOT( slotSelectionChanged( const QItemSelection &, const QItemSelection & ) ) );
 }
 
@@ -781,11 +960,6 @@ void PerformanceStatusTreeView::slotSelectionChanged( const QItemSelection&, con
     m_chart->model()->setNodes( nodes );
 }
 
-void PerformanceStatusTreeView::draw()
-{
-    m_chart->draw();
-}
-
 NodeItemModel *PerformanceStatusTreeView::nodeModel() const
 {
     return static_cast<NodeItemModel*>( m_tree->model() );
@@ -811,7 +985,7 @@ void PerformanceStatusTreeView::setProject( Project *project )
 bool PerformanceStatusTreeView::loadContext( const KoXmlElement &context )
 {
     kDebug();
-    
+
     bool res = false;
     res = m_chart->loadContext( context.namedItem( "chart" ).toElement() );
     res &= m_tree->loadContext( nodeModel()->columnMap(), context.namedItem( "tree" ).toElement() );
@@ -823,7 +997,7 @@ void PerformanceStatusTreeView::saveContext( QDomElement &context ) const
     QDomElement c = context.ownerDocument().createElement( "chart" );
     context.appendChild( c );
     m_chart->saveContext( c );
-    
+
     QDomElement t = context.ownerDocument().createElement( "tree" );
     context.appendChild( t );
     m_tree->saveContext( nodeModel()->columnMap(), t );
@@ -838,7 +1012,7 @@ PerformanceStatusView::PerformanceStatusView( KoDocument *part, QWidget *parent
     l->setMargin( 0 );
     m_view = new PerformanceStatusTreeView( this );
     l->addWidget( m_view );
-    
+
     setupGui();
 
     connect( m_view->treeView(), SIGNAL( headerContextMenuRequested( const QPoint& ) ), SLOT( slotHeaderContextMenuRequested( const QPoint& ) ) );
@@ -862,7 +1036,7 @@ void PerformanceStatusView::slotContextMenuRequested( const QModelIndex &index,
     slotContextMenuRequested( node, pos );
 }
 
-Node *PerformanceStatusView::currentNode() const 
+Node *PerformanceStatusView::currentNode() const
 {
     return m_view->nodeModel()->node( m_view->treeView()->selectionModel()->currentIndex() );
 }
@@ -904,11 +1078,6 @@ void PerformanceStatusView::setProject( Project *project )
     m_view->setProject( project );
 }
 
-void PerformanceStatusView::draw()
-{
-    m_view->draw();
-}
-
 void PerformanceStatusView::setGuiActive( bool activate )
 {
     kDebug()<<activate;
@@ -949,37 +1118,45 @@ PerformanceStatusViewSettingsPanel::PerformanceStatusViewSettingsPanel( Performa
     m_view( view )
 {
     setupUi( this );
-    
-    NodeChartModel::DataShown show = m_view->dataShown();
 
-    ui_bcwsCost->setCheckState( show.showBCWSCost ? Qt::Checked : Qt::Unchecked );
-    ui_bcwpCost->setCheckState( show.showBCWPCost ? Qt::Checked : Qt::Unchecked );
-    ui_acwpCost->setCheckState( show.showACWPCost ? Qt::Checked : Qt::Unchecked );
-    ui_cost->setChecked( show.showCost );
+    PerformanceChartInfo info = m_view->chartInfo();
 
-    ui_bcwsEffort->setCheckState( show.showBCWSEffort ? Qt::Checked : Qt::Unchecked );
-    ui_bcwpEffort->setCheckState( show.showBCWPEffort ? Qt::Checked : Qt::Unchecked );
-    ui_acwpEffort->setCheckState( show.showACWPEffort ? Qt::Checked : Qt::Unchecked );
-    ui_effort->setChecked( show.showEffort );
+    ui_linechart->setChecked( info.showLineChart );
+    ui_barchart->setChecked( info.showBarChart );
+
+    ui_bcwsCost->setCheckState( info.showBCWSCost ? Qt::Checked : Qt::Unchecked );
+    ui_bcwpCost->setCheckState( info.showBCWPCost ? Qt::Checked : Qt::Unchecked );
+    ui_acwpCost->setCheckState( info.showACWPCost ? Qt::Checked : Qt::Unchecked );
+    ui_cost->setChecked( info.showCost );
+
+    ui_bcwsEffort->setCheckState( info.showBCWSEffort ? Qt::Checked : Qt::Unchecked );
+    ui_bcwpEffort->setCheckState( info.showBCWPEffort ? Qt::Checked : Qt::Unchecked );
+    ui_acwpEffort->setCheckState( info.showACWPEffort ? Qt::Checked : Qt::Unchecked );
+    ui_effort->setChecked( info.showEffort );
 }
 
 void PerformanceStatusViewSettingsPanel::slotOk()
 {
-    NodeChartModel::DataShown show;
-    show.showBCWSCost = ui_bcwsCost->checkState() == Qt::Unchecked ? false : true;
-    show.showBCWPCost = ui_bcwpCost->checkState() == Qt::Unchecked ? false : true;
-    show.showACWPCost = ui_acwpCost->checkState() == Qt::Unchecked ? false : true;
-    show.showCost = ui_cost->isChecked();
+    PerformanceChartInfo info;
+    info.showLineChart = ui_linechart->isChecked();
+    info.showBarChart = ui_barchart->isChecked();
 
-    show.showBCWSEffort = ui_bcwsEffort->checkState() == Qt::Unchecked ? false : true;
-    show.showBCWPEffort = ui_bcwpEffort->checkState() == Qt::Unchecked ? false : true;
-    show.showACWPEffort = ui_acwpEffort->checkState() == Qt::Unchecked ? false : true;
-    show.showEffort = ui_effort->isChecked();
-    m_view->setDataShown( show );
+    info.showBCWSCost = ui_bcwsCost->checkState() == Qt::Unchecked ? false : true;
+    info.showBCWPCost = ui_bcwpCost->checkState() == Qt::Unchecked ? false : true;
+    info.showACWPCost = ui_acwpCost->checkState() == Qt::Unchecked ? false : true;
+    info.showCost = ui_cost->isChecked();
+
+    info.showBCWSEffort = ui_bcwsEffort->checkState() == Qt::Unchecked ? false : true;
+    info.showBCWPEffort = ui_bcwpEffort->checkState() == Qt::Unchecked ? false : true;
+    info.showACWPEffort = ui_acwpEffort->checkState() == Qt::Unchecked ? false : true;
+    info.showEffort = ui_effort->isChecked();
+    m_view->setChartInfo( info );
 }
 
 void PerformanceStatusViewSettingsPanel::setDefault()
 {
+    ui_linechart->setChecked( true );
+
     ui_bcwsCost->setCheckState( Qt::Checked );
     ui_bcwpCost->setCheckState( Qt::Checked );
     ui_acwpCost->setCheckState( Qt::Checked );
@@ -999,7 +1176,7 @@ PerformanceStatusViewSettingsDialog::PerformanceStatusViewSettingsDialog( Perfor
     KPageWidgetItem *page = insertWidget( 0, panel, i18n( "Chart" ), i18n( "Chart Settings" ) );
     setCurrentPage( page );
     //connect( panel, SIGNAL( changed( bool ) ), this, SLOT( enableButtonOk( bool ) ) );
-    
+
     connect( this, SIGNAL( okClicked() ), panel, SLOT( slotOk() ) );
     connect( this, SIGNAL( defaultClicked() ), panel, SLOT( setDefault() ) );
 }
@@ -1014,7 +1191,7 @@ ProjectStatusViewSettingsDialog::ProjectStatusViewSettingsDialog( PerformanceSta
     addPage( page );
 
     //connect( panel, SIGNAL( changed( bool ) ), this, SLOT( enableButtonOk( bool ) ) );
-    
+
     connect( this, SIGNAL( okClicked() ), panel, SLOT( slotOk() ) );
     connect( this, SIGNAL( defaultClicked() ), panel, SLOT( setDefault() ) );
 }
