@@ -36,6 +36,7 @@
 #include <QTimer>
 
 // KDE
+#include <krun.h>
 #include <kimageio.h>
 #include <kfiledialog.h>
 #include <kglobal.h>
@@ -44,6 +45,7 @@
 #include <kmessagebox.h>
 #include <kactioncollection.h>
 #include <KUndoStack>
+#include <kstandarddirs.h>
 
 // KOffice
 #include <KoApplication.h>
@@ -115,7 +117,9 @@ public:
             , macroNestDepth(0)
             , ioProgressTotalSteps(0)
             , ioProgressBase(0)
-            , kraLoader(0) {
+            , kraLoader(0)
+            , dieOnError(false)
+    {
     }
 
     ~KisDocPrivate() {
@@ -137,6 +141,9 @@ public:
     KisKraLoader* kraLoader;
     KisKraSaver* kraSaver;
 
+    QString error;
+    bool dieOnError;
+
 };
 
 
@@ -144,12 +151,13 @@ KisDoc2::KisDoc2(QWidget *parentWidget, QObject *parent, bool singleViewMode)
         : KoDocument(parentWidget, parent, singleViewMode)
         , m_d(new KisDocPrivate())
 {
-
     setComponentData(KisFactory2::componentData(), false);
     setTemplateType("krita_template");
     init();
     connect(this, SIGNAL(sigLoadingFinished()), this, SLOT(slotLoadingFinished()));
     undoStack()->setUndoLimit(KisConfig().undoStackLimit());
+    setBackupFile(KisConfig().backupFile());
+
 }
 
 KisDoc2::~KisDoc2()
@@ -157,7 +165,7 @@ KisDoc2::~KisDoc2()
     // Despite being QObject they needs to be deleted before the image
     delete m_d->shapeController;
     delete m_d->nodeModel;
-    
+
     // The following line trigger the deletion of the image
     m_d->image.clear();
 
@@ -170,25 +178,22 @@ QByteArray KisDoc2::mimeType() const
 }
 
 void KisDoc2::slotLoadingFinished() {
+    image()->refreshGraph();
     setAutoSave(KisConfig().autoSaveInterval());
 }
 
 void KisDoc2::openExistingFile(const KUrl& url)
 {
-    setUndo(false);
     qApp->setOverrideCursor(Qt::BusyCursor);
     KoDocument::openExistingFile(url);
     qApp->restoreOverrideCursor();
-    setUndo(true);
 }
 
 void KisDoc2::openTemplate(const KUrl& url)
 {
-    setUndo(false);
     qApp->setOverrideCursor(Qt::BusyCursor);
     KoDocument::openTemplate(url);
     qApp->restoreOverrideCursor();
-    setUndo(true);
 }
 
 bool KisDoc2::init()
@@ -207,8 +212,6 @@ bool KisDoc2::init()
     connect(undoStack(), SIGNAL(indexChanged(int)), SLOT(undoIndexChanged(int)));
     Q_CHECK_PTR(m_d->undoAdapter);
 
-    setUndo(true);
-
     m_d->nserver = new KisNameServer(1);
     Q_CHECK_PTR(m_d->nserver);
 
@@ -271,7 +274,6 @@ bool KisDoc2::loadXML(const KoXmlDocument& doc, KoStore *)
         return false;
     }
 
-    setUndo(false);
     Q_ASSERT(m_d->kraLoader == 0);
     m_d->kraLoader = new KisKraLoader(this, syntaxVersion);
 
@@ -321,7 +323,6 @@ bool KisDoc2::completeLoading(KoStore *store)
     m_d->kraLoader = 0;
 
     setModified(false);
-    setUndo(true);
     m_d->image->setUndoAdapter(m_d->undoAdapter);
     m_d->shapeController->setImage(m_d->image);
     m_d->nodeModel->setImage(m_d->image);
@@ -350,7 +351,7 @@ QList<KoDocument::CustomDocumentWidgetItem> KisDoc2::createCustomDocumentWidgets
 
     QList<KoDocument::CustomDocumentWidgetItem> widgetList;
     KoDocument::CustomDocumentWidgetItem item;
-    item.widget = new KisCustomImageWidget(parent, this, w, h, clipAvailable, cfg.defImageResolution(), cfg.workingColorSpace(), "unnamed");
+    item.widget = new KisCustomImageWidget(parent, this, w, h, clipAvailable, cfg.defImageResolution(), cfg.defColorModel(), cfg.defColorDepth(), cfg.defColorProfile(), "unnamed");
     widgetList << item;
 
     return widgetList;
@@ -389,15 +390,13 @@ bool KisDoc2::newImage(const QString& name,
 
     qApp->setOverrideCursor(Qt::BusyCursor);
 
-    setUndo(false);
-
     image = new KisImage(m_d->undoAdapter, width, height, cs, name);
     Q_CHECK_PTR(image);
     image->lock();
 
     connect(image.data(), SIGNAL(sigImageModified()), this, SLOT(setModified()));
     image->setResolution(imageResolution, imageResolution);
-    image->setProfile(cs->profile());
+    image->assignImageProfile(cs->profile());
     documentInfo()->setAboutInfo("title", name);
     documentInfo()->setAboutInfo("comments", description);
 
@@ -413,8 +412,9 @@ bool KisDoc2::newImage(const QString& name,
     cfg.defImageWidth(width);
     cfg.defImageHeight(height);
     cfg.defImageResolution(imageResolution);
-
-    setUndo(true);
+    cfg.defColorModel(image->colorSpace()->colorModelId().id());
+    cfg.defColorDepth(image->colorSpace()->colorDepthId().id());
+    cfg.defColorProfile(image->colorSpace()->profile()->name());
 
     qApp->restoreOverrideCursor();
     return true;
@@ -443,19 +443,38 @@ void KisDoc2::showStartUpWidget(KoMainWindow* parent, bool alwaysShow)
     // print error if the lcms engine is not available
     if (!KoColorSpaceEngineRegistry::instance()->contains("icc")) {
         // need to wait 1 event since exiting here would not work.
+        m_d->error = i18n("The KOffice LittleCMS color management plugin is not installed. Krita will quit now.");
+        m_d->dieOnError = true;
         QTimer::singleShot(0, this, SLOT(showErrorAndDie()));
     }
-    else {
-        KoDocument::showStartUpWidget(parent, alwaysShow);
+
+    KoDocument::showStartUpWidget(parent, alwaysShow);
+    KisConfig cfg;
+    if (cfg.firstRun()) {
+
+        QStringList qtversion = QString(qVersion()).split('.');
+        if (qtversion[0] == "4" && qtversion[1] <= "6" && qtversion[2].toInt() < 3) {
+            m_d->error = i18n("Krita needs at least Qt 4.6.3 to work correctly. Your Qt version is %1. If you have a graphics tablet it will not work correctly!", qVersion());
+            m_d->dieOnError = false;
+            QTimer::singleShot(0, this, SLOT(showErrorAndDie()));
+        }
+
+        QString fname = KisFactory2::componentData().dirs()->findResource("kis_images", "krita_first_start.kra");
+        if (!fname.isEmpty()) {
+            openUrl(fname);
+        }
+        cfg.setFirstRun(false);
     }
 }
 
 void KisDoc2::showErrorAndDie()
 {
     KMessageBox::error(widget(),
-                       i18n("The KOffice LittleCMS color management plugin is not installed. Krita will quit now."),
-                       i18n("Installation Error"));
-    QCoreApplication::exit(10);
+                       m_d->error,
+                       i18n("Installation error"));
+    if (m_d->dieOnError) {
+        exit(10);
+    }
 }
 
 void KisDoc2::paintContent(QPainter& painter, const QRect& rc)
@@ -480,16 +499,6 @@ QPixmap KisDoc2::generatePreview(const QSize& size)
     return QPixmap(size);
 }
 
-void KisDoc2::setUndo(bool undo)
-{
-    m_d->undoAdapter->setUndo(undo);
-}
-
-bool KisDoc2::undo() const
-{
-    return m_d->undoAdapter->undo();
-}
-
 KoShapeControllerBase * KisDoc2::shapeController() const
 {
     return m_d->shapeController;
@@ -516,7 +525,6 @@ void KisDoc2::prepareForImport()
 {
     if (m_d->nserver == 0)
         init();
-    setUndo(false);
 }
 
 KisImageWSP KisDoc2::image() const
@@ -536,7 +544,6 @@ void KisDoc2::setCurrentImage(KisImageWSP image)
     m_d->shapeController->setImage(image);
     m_d->nodeModel->setImage(image);
 
-    setUndo(true);
     setModified(false);
 
     connect(m_d->image, SIGNAL(sigImageModified()), this, SLOT(setModified()));
