@@ -21,6 +21,8 @@
 #include <iostream>
 
 #include <QMimeData>
+#include <QDataStream>
+#include <QBuffer>
 
 #include <klocale.h>
 
@@ -36,12 +38,15 @@
 
 #include "kis_config.h"
 #include "kis_config_notifier.h"
+#include <QTimer>
 
 class KisNodeModel::Private
 {
 public:
     KisImageWSP image;
     bool showRootLayer;
+    QList<KisNode*> updateQueue;
+    QTimer* updateTimer;
 };
 
 KisNodeModel::KisNodeModel(QObject * parent)
@@ -50,6 +55,8 @@ KisNodeModel::KisNodeModel(QObject * parent)
 {
     updateSettings();
     connect(KisConfigNotifier::instance(), SIGNAL(configChanged()), this, SLOT(updateSettings()));
+    m_d->updateTimer = new QTimer(this);
+    connect(m_d->updateTimer, SIGNAL(timeout()), SLOT(updateNodes()));
 }
 
 KisNodeModel::~KisNodeModel()
@@ -74,6 +81,8 @@ void KisNodeModel::setImage(KisImageWSP image)
             SLOT(beginRemoveNodes(KisNode*, int)));
     connect(m_d->image, SIGNAL(sigNodeHasBeenRemoved(KisNode*, int)),
             SLOT(endRemoveNodes(KisNode*, int)));
+    connect(m_d->image, SIGNAL(sigNodeChanged(KisNode*)),
+            SLOT(nodeChanged(KisNode*)));
 }
 
 KisNodeSP KisNodeModel::nodeFromIndex(const QModelIndex &index)
@@ -354,6 +363,9 @@ void KisNodeModel::endInsertNodes(KisNode * parent, int index)
 
 void KisNodeModel::beginRemoveNodes(KisNode * parent, int index)
 {
+    m_d->updateTimer->stop();
+    m_d->updateQueue.clear();
+    
     //dbgUI <<"KisNodeModel::beginRemoveNodes parent=" << parent << ", index=" << index;
     beginRemoveRows(indexFromNode(parent), parent->childCount() - 1 - index, parent->childCount() - 1 - index);
 }
@@ -381,7 +393,9 @@ QMimeData * KisNodeModel::mimeData(const QModelIndexList & indexes) const
     //dbgUI <<"KisNodeModel::mimeData";
     QMimeData* data = new QMimeData;
     QByteArray encoded;
-    QDataStream stream(&encoded, QIODevice::WriteOnly);
+    QBuffer buf(&encoded);
+    buf.open(QIODevice::WriteOnly);
+    QDataStream stream(&buf);
 
     // encode the data
     QModelIndexList::ConstIterator it = indexes.begin();
@@ -389,6 +403,8 @@ QMimeData * KisNodeModel::mimeData(const QModelIndexList & indexes) const
         stream << qVariantFromValue(qulonglong(it->internalPointer()));
     }
 
+    buf.close();
+
     data->setData("application/x-kritalayermodeldatalist", encoded);
     return data;
 }
@@ -404,7 +420,7 @@ bool KisNodeModel::dropMimeData(const QMimeData * data, Qt::DropAction action, i
         return false;
     }
     QByteArray encoded = data->data("application/x-kritalayermodeldatalist");
-    QDataStream stream(&encoded, QIODevice::ReadOnly);
+    QDataStream stream(encoded);
     QList<KisNode*> nodes;
     while (! stream.atEnd()) {
         QVariant v;
@@ -475,5 +491,22 @@ void KisNodeModel::layersChanged()
     reset();
 }
 
+void KisNodeModel::nodeChanged(KisNode* node)
+{
+    if (!m_d->updateQueue.contains(node)) {
+        m_d->updateQueue.append(node);
+    }
+    m_d->updateTimer->start(1000);
+}
+
+void KisNodeModel::updateNodes()
+{
+    foreach(KisNode * node, m_d->updateQueue) {
+        QModelIndex index = indexFromNode(node);
+        emit dataChanged(index, index);
+    }
+    m_d->updateTimer->stop();
+    m_d->updateQueue.clear();
+}
 
 #include "kis_node_model.moc"
