@@ -5,7 +5,7 @@
  *                1999 Michael Koch    <koch@kde.org>
  *                1999 Carsten Pfeiffer <pfeiffer@kde.org>
  *                2002 Patrick Julien <freak@codepimps.org>
- *                2003-2007 Boudewijn Rempt <boud@valdyas.org>
+ *                2003-20010 Boudewijn Rempt <boud@valdyas.org>
  *                2004 Clarence Dang <dang@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -38,7 +38,6 @@
 #include <QPrintDialog>
 #include <QObject>
 
-#include <k3urldrag.h>
 #include <kaction.h>
 #include <klocale.h>
 #include <kmenu.h>
@@ -53,9 +52,10 @@
 #include <kxmlguiwindow.h>
 #include <kxmlguifactory.h>
 #include <kmessagebox.h>
+#include <ktemporaryfile.h>
+#include <kactioncollection.h>
 
 #include <KoMainWindow.h>
-#include <KoCanvasController.h>
 #include <KoSelection.h>
 #include <KoToolBoxFactory.h>
 #include <KoZoomHandler.h>
@@ -66,8 +66,9 @@
 #include <KoDockRegistry.h>
 #include <KoResourceServerProvider.h>
 #include <KoCompositeOp.h>
-
-#include <kactioncollection.h>
+#include <KoTemplateCreateDia.h>
+#include <KoCanvasControllerWidget.h>
+#include <KoDocumentEntry.h>
 
 #include <kis_image.h>
 #include <kis_undo_adapter.h>
@@ -77,6 +78,8 @@
 #include "kis_config_notifier.h"
 #include "kis_statusbar.h"
 #include "canvas/kis_canvas2.h"
+#include "canvas/kis_canvas_controller.h"
+#include "kis_coordinates_converter.h"
 #include "kis_doc2.h"
 #include "kis_factory2.h"
 #include "kis_filter_manager.h"
@@ -112,6 +115,22 @@
 #include "ko_favorite_resource_manager.h"
 #include "kis_paintop_box.h"
 
+class BlockingUserInputEventFilter : public QObject
+{
+    bool eventFilter(QObject *watched, QEvent *event)
+    {
+        Q_UNUSED(watched);
+        if(dynamic_cast<QWheelEvent*>(event)
+            || dynamic_cast<QKeyEvent*>(event)
+            || dynamic_cast<QMouseEvent*>(event)) {
+            return true;
+        }
+        else {
+            return false;
+        }
+    }
+};
+
 class KisView2::KisView2Private
 {
 
@@ -162,7 +181,8 @@ public:
     KisFilterManager * filterManager;
     KisStatusBar * statusBar;
     KAction * totalRefresh;
-    KAction* toggleDockers;
+    KAction* mirrorCanvas;
+    KAction* createTemplate;
     KisSelectionManager *selectionManager;
     KisControlFrame * controlFrame;
     KisNodeManager * nodeManager;
@@ -172,6 +192,7 @@ public:
     KisPerspectiveGridManager * perspectiveGridManager;
     KisPaintingAssistantsManager* paintingAssistantManager;
     KoFavoriteResourceManager* favoriteResourceManager;
+    BlockingUserInputEventFilter blockingEventFilter;
 };
 
 
@@ -187,17 +208,13 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     m_d->totalRefresh->setShortcut(QKeySequence(Qt::CTRL + Qt::SHIFT + Qt::Key_R));
     connect(m_d->totalRefresh, SIGNAL(triggered()), this, SLOT(slotTotalRefresh()));
 
-    if (shell()) {
-        m_d->toggleDockers = new KToggleAction(i18n("Show Dockers"), this);
-        m_d->toggleDockers->setChecked(true);
-        actionCollection()->addAction("toggledockers", m_d->toggleDockers);
+    m_d->createTemplate = new KAction( i18n( "&Create Template From Image..." ), this);
+    actionCollection()->addAction("createTemplate", m_d->createTemplate);
+    connect(m_d->createTemplate, SIGNAL(triggered()), this, SLOT(slotCreateTemplate()));
 
-
-        m_d->toggleDockers->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
-        connect(m_d->toggleDockers, SIGNAL(toggled(bool)), shell(), SLOT(toggleDockersVisibility(bool)));
-    } else {
-        m_d->toggleDockers = 0;
-    }
+    KAction *firstRun = new KAction( i18n( "Load Tutorial"), this);
+    actionCollection()->addAction("first_run", firstRun);
+    connect(firstRun, SIGNAL(triggered()), this, SLOT(slotFirstRun()));
 
     setComponentData(KisFactory2::componentData(), false);
 
@@ -214,10 +231,17 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     m_d->doc = doc;
     m_d->viewConverter = new KoZoomHandler();
 
-    m_d->canvasController = new KoCanvasController(this);
-    m_d->canvasController->setDrawShadow(false);
-    m_d->canvasController->setMargin(10);
-    m_d->canvasController->setCanvasMode(KoCanvasController::Infinite);
+    KoCanvasControllerWidget *canvasController = new KisCanvasController(this);
+    canvasController->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    canvasController->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOn);
+    canvasController->setDrawShadow(false);
+    canvasController->setCanvasMode(KoCanvasController::Infinite);
+    KisConfig cfg;
+    canvasController->setZoomWithWheel(cfg.zoomWithWheel());
+    canvasController->setVastScrolling(cfg.vastScrolling());
+
+    m_d->canvasController = canvasController;
+
 
     m_d->canvas = new KisCanvas2(m_d->viewConverter, this, doc->shapeController());
     m_d->canvasController->setCanvas(m_d->canvas);
@@ -235,18 +259,39 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
     createManagers();
     createActions();
 
+    m_d->mirrorCanvas = new KToggleAction(i18n("Mirror Image"), this);
+    m_d->mirrorCanvas->setChecked(false);
+    actionCollection()->addAction("mirror_canvas", m_d->mirrorCanvas);
+
+    m_d->mirrorCanvas->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_I));
+    connect(m_d->mirrorCanvas, SIGNAL(toggled(bool)),m_d->canvas, SLOT(mirrorCanvas(bool)));
+
+    KAction *rotateCanvasRight = new KAction(i18n("Rotate Canvas Right"), this);
+    actionCollection()->addAction("rotate_canvas_right", rotateCanvasRight);
+    rotateCanvasRight->setShortcut(QKeySequence("Ctrl+]"));
+    connect(rotateCanvasRight, SIGNAL(triggered()),m_d->canvas, SLOT(rotateCanvasRight15()));
+
+    KAction *rotateCanvasLeft = new KAction(i18n("Rotate Canvas Left"), this);
+    actionCollection()->addAction("rotate_canvas_left", rotateCanvasLeft);
+    rotateCanvasLeft->setShortcut(QKeySequence("Ctrl+["));
+    connect(rotateCanvasLeft, SIGNAL(triggered()),m_d->canvas, SLOT(rotateCanvasLeft15()));
+
+    KAction *resetCanvasTransformations = new KAction(i18n("Reset Canvas Transformations"), this);
+    actionCollection()->addAction("reset_canvas_transformations", resetCanvasTransformations);
+    resetCanvasTransformations->setShortcut(QKeySequence("Ctrl+'"));
+    connect(resetCanvasTransformations, SIGNAL(triggered()),m_d->canvas, SLOT(resetCanvasTransformations()));
 
     if (shell())
     {
         KoToolBoxFactory toolBoxFactory(m_d->canvasController, i18n("Tools"));
         shell()->createDockWidget(&toolBoxFactory);
 
-        connect(m_d->canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
+        connect(canvasController, SIGNAL(toolOptionWidgetsChanged(const QMap<QString, QWidget *> &, QWidget*)),
                 shell()->dockerManager(), SLOT(newOptionWidgets(const  QMap<QString, QWidget *> &, QWidget*)));
     }
 
     m_d->statusBar = new KisStatusBar(this);
-    connect(m_d->canvasController, SIGNAL(documentMousePositionChanged(const QPointF &)),
+    connect(m_d->canvasController->proxyObject, SIGNAL(documentMousePositionChanged(const QPointF &)),
             m_d->statusBar, SLOT(documentMousePositionChanged(const QPointF &)));
 
     connect(m_d->nodeManager, SIGNAL(sigNodeActivated(KisNodeSP)),
@@ -276,6 +321,7 @@ KisView2::KisView2(KisDoc2 * doc, QWidget * parent)
 
     // canvas sends signal that origin is changed
     connect(m_d->canvas, SIGNAL(documentOriginChanged()), m_d->zoomManager, SLOT(pageOffsetChanged()));
+    connect(m_d->canvas, SIGNAL(scrollAreaSizeChanged()), m_d->zoomManager, SLOT(slotScrollAreaSizeChanged()));
 
     setAcceptDrops(true);
 }
@@ -292,9 +338,7 @@ void KisView2::dragEnterEvent(QDragEnterEvent *event)
     dbgUI << "KisView2::dragEnterEvent";
     // Only accept drag if we're not busy, particularly as we may
     // be showing a progress bar and calling qApp->processEvents().
-    if (K3URLDrag::canDecode(event) && QApplication::overrideCursor() == 0) {
-        event->accept();
-    } else if (event->mimeData()->hasImage()) {
+    if (event->mimeData()->hasImage()) {
         event->accept();
     } else {
         event->ignore();
@@ -331,55 +375,53 @@ void KisView2::dropEvent(QDropEvent *event)
         return;
     }
 
-    KUrl::List urls;
-    if (K3URLDrag::decode(event, urls)) {
-        if (urls.count() > 0) {
+    QList<QUrl> urls = event->mimeData()->urls();
+    if (urls.length() > 0) {
 
-            KMenu popup;
-            popup.setObjectName("drop_popup");
+        KMenu popup;
+        popup.setObjectName("drop_popup");
 
-            QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
-            QAction *insertAsNewLayers = new QAction(i18n("Insert as New Layers"), &popup);
+        QAction *insertAsNewLayer = new QAction(i18n("Insert as New Layer"), &popup);
+        QAction *insertAsNewLayers = new QAction(i18n("Insert as New Layers"), &popup);
 
-            QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
-            QAction *openInNewDocuments = new QAction(i18n("Open in New Documents"), &popup);
+        QAction *openInNewDocument = new QAction(i18n("Open in New Document"), &popup);
+        QAction *openInNewDocuments = new QAction(i18n("Open in New Documents"), &popup);
 
-            QAction *cancel = new QAction(i18n("Cancel"), &popup);
+        QAction *cancel = new QAction(i18n("Cancel"), &popup);
 
-            if (urls.count() == 1) {
-                if (!image().isNull()) {
-                    popup.addAction(insertAsNewLayer);
-                }
-                popup.addAction(openInNewDocument);
-            } else {
-                if (!image().isNull()) {
-                    popup.addAction(insertAsNewLayers);
-                }
-                popup.addAction(openInNewDocuments);
+        if (urls.count() == 1) {
+            if (!image().isNull()) {
+                popup.addAction(insertAsNewLayer);
             }
+            popup.addAction(openInNewDocument);
+        } else {
+            if (!image().isNull()) {
+                popup.addAction(insertAsNewLayers);
+            }
+            popup.addAction(openInNewDocuments);
+        }
 
-            popup.addSeparator();
-            popup.addAction(cancel);
+        popup.addSeparator();
+        popup.addAction(cancel);
 
-            QAction *action = popup.exec(QCursor::pos());
+        QAction *action = popup.exec(QCursor::pos());
 
-            if (action != 0 && action != cancel) {
-                for (KUrl::List::ConstIterator it = urls.constBegin(); it != urls.constEnd(); ++it) {
-                    KUrl url = *it;
+        if (action != 0 && action != cancel) {
+            foreach(QUrl url, urls) {
 
-                    if (action == insertAsNewLayer || action == insertAsNewLayers) {
-                        m_d->imageManager->importImage(url);
-                    } else {
-                        Q_ASSERT(action == openInNewDocument || action == openInNewDocuments);
+                if (action == insertAsNewLayer || action == insertAsNewLayers) {
+                    m_d->imageManager->importImage(KUrl(url));
+                } else {
+                    Q_ASSERT(action == openInNewDocument || action == openInNewDocuments);
 
-                        if (shell() != 0) {
-                            shell()->openDocument(url);
-                        }
+                    if (shell() != 0) {
+                        shell()->openDocument(url);
                     }
                 }
             }
         }
     }
+
 }
 
 KoZoomController *KisView2::zoomController() const
@@ -503,16 +545,19 @@ KisUndoAdapter * KisView2::undoAdapter()
 
 void KisView2::slotLoadingFinished()
 {
-    image()->refreshGraph();
-
+    /**
+     * Cold-start of image-size signals
+     */
     slotImageSizeChanged();
-
     if (m_d->statusBar) {
         m_d->statusBar->imageSizeChanged(image()->width(), image()->height());
     }
-    m_d->resourceProvider->slotImageSizeChanged();
-
-    m_d->nodeManager->nodesUpdated();
+    if (m_d->resourceProvider) {
+        m_d->resourceProvider->slotImageSizeChanged();
+    }
+    if (m_d->nodeManager) {
+        m_d->nodeManager->nodesUpdated();
+    }
 
     connectCurrentImage();
 
@@ -530,8 +575,10 @@ void KisView2::slotLoadingFinished()
     /**
      * Dirty hack alert
      */
-    m_d->viewConverter->setZoomMode(KoZoomMode::ZOOM_PAGE);
-    m_d->zoomManager->zoomController()->setAspectMode(true);
+    if (m_d->viewConverter)
+        m_d->viewConverter->setZoomMode(KoZoomMode::ZOOM_PAGE);
+    if (m_d->zoomManager && m_d->zoomManager->zoomController())
+        m_d->zoomManager->zoomController()->setAspectMode(true);
 
     updateGUI();
 
@@ -607,7 +654,7 @@ void KisView2::connectCurrentImage()
     if (image()) {
         if (m_d->statusBar) {
             connect(image(), SIGNAL(sigColorSpaceChanged(const KoColorSpace *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
-            connect(image(), SIGNAL(sigProfileChanged(KoColorProfile *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
+            connect(image(), SIGNAL(sigProfileChanged(const KoColorProfile *)), m_d->statusBar, SLOT(updateStatusBarProfileLabel()));
             connect(image(), SIGNAL(sigSizeChanged(qint32, qint32)), m_d->statusBar, SLOT(imageSizeChanged(qint32, qint32)));
 
         }
@@ -648,6 +695,8 @@ void KisView2::slotPreferences()
     if (KisDlgPreferences::editPreferences()) {
         KisConfigNotifier::instance()->notifyConfigChanged();
         m_d->resourceProvider->resetDisplayProfile();
+        KisConfig cfg;
+        static_cast<KoCanvasControllerWidget*>(m_d->canvasController)->setZoomWithWheel(cfg.zoomWithWheel());
 
         // Update the settings for all nodes -- they don't query
         // KisConfig directly because they need the settings during
@@ -678,15 +727,13 @@ void KisView2::slotImageSizeChanged()
     m_d->zoomManager->zoomController()->setPageSize(size);
     m_d->zoomManager->zoomController()->setDocumentSize(size);
 
-    QSize documentSize(int(ceil(m_d->viewConverter->documentToViewX(image()->width()  / image()->xRes()))),
-                       int(ceil(m_d->viewConverter->documentToViewY(image()->height() / image()->yRes()))));
-    m_d->canvasController->setDocumentSize(documentSize);
+    canvasBase()->notifyZoomChanged();
+    QSize widgetSize = canvasBase()->coordinatesConverter()->imageRectInWidgetPixels().toAlignedRect().size();
+    m_d->canvasController->updateDocumentSize(widgetSize, true);
 
     m_d->zoomManager->updateGUI();
 
     //update view
-    canvasBase()->preScale();
-    canvasBase()->adjustOrigin();
     canvas()->update();
 }
 
@@ -696,18 +743,19 @@ void KisView2::loadPlugins()
     // Load all plugins
     KService::List offers = KServiceTypeTrader::self()->query(QString::fromLatin1("Krita/ViewPlugin"),
                                                               QString::fromLatin1("(Type == 'Service') and "
-                                                                                  "([X-Krita-Version] == 3)"));
+                                                                                  "([X-Krita-Version] == 4)"));
     KService::List::ConstIterator iter;
     for (iter = offers.constBegin(); iter != offers.constEnd(); ++iter) {
         KService::Ptr service = *iter;
         dbgUI << "Load plugin " << service->name();
-        int errCode = 0;
+        QString error;
+
         KParts::Plugin* plugin =
-                KService::createInstance<KParts::Plugin> (service, this, QStringList(), &errCode);
+                dynamic_cast<KParts::Plugin*>(service->createInstance<QObject>(this, QVariantList(), &error));
         if (plugin) {
             insertChildClient(plugin);
         } else {
-            errKrita << "Fail to create an instance for " << service->name() << " " << KLibLoader::errorString(errCode) << " " << KLibLoader::self()->lastErrorMessage();
+            errKrita << "Fail to create an instance for " << service->name() << " " << error;
         }
     }
 }
@@ -748,16 +796,68 @@ void KisView2::slotTotalRefresh()
     m_d->canvas->resetCanvas(cfg.useOpenGL());
 }
 
-void KisView2::resizeEvent ( QResizeEvent * event )
+void KisView2::slotCreateTemplate()
 {
-    dbgUI << "resize: " << event->oldSize() << " to " << event->size() << "main window" << mainWindow()->size();
+    int width = 60;
+    int height = 60;
+    QPixmap pix = m_d->doc->generatePreview(QSize(width, height));
+
+    KTemporaryFile tempFile;
+    tempFile.setSuffix(".kra");
+
+    //Check that creation of temp file was successful
+    if (!tempFile.open()) {
+        qWarning("Creation of temporary file to store template failed.");
+        return;
+    }
+
+    m_d->doc->saveNativeFormat(tempFile.fileName());
+
+    KoTemplateCreateDia::createTemplate("krita_template", KisFactory2::componentData(),
+                                        tempFile.fileName(), pix, this);
+
+    KisFactory2::componentData().dirs()->addResourceType("krita_template", "data", "krita/templates/");
 
-//     // This code makes the full screen mode unusuable, it force the size of the view, (the reason is that QApplication::desktop()->availableGeometry(this) return the size of the screen minus the size taken by a panel, while in full screen mode the window cover a bigger area), and further more it does not prevent my window to increase its size out of the screen
-//     if (mainWindow()->size().height() > QApplication::desktop()->availableGeometry(this).height()) {
-//         mainWindow()->resize(mainWindow()->width(),
-//                              QApplication::desktop()->availableGeometry(this).height());
-//     }
 }
 
+void KisView2::disableControls()
+{
+    // prevents possible crashes, if somebody changes the paintop during dragging by using the mousewheel
+    // this is for Bug 250944
+    // the solution blocks all wheel, mouse and key event, while dragging with the freehand tool
+    // see KisToolFreehand::initPaint() and endPaint()
+    m_d->controlFrame->paintopBox()->installEventFilter(&m_d->blockingEventFilter);
+    foreach(QObject* child, m_d->controlFrame->paintopBox()->children()) {
+        child->installEventFilter(&m_d->blockingEventFilter);
+    }
+}
+
+void KisView2::enableControls()
+{
+    m_d->controlFrame->paintopBox()->removeEventFilter(&m_d->blockingEventFilter);
+    foreach(QObject* child, m_d->controlFrame->paintopBox()->children()) {
+        child->removeEventFilter(&m_d->blockingEventFilter);
+    }
+}
+
+void KisView2::slotFirstRun()
+{
+    QString fname = KisFactory2::componentData().dirs()->findResource("kis_images", "krita_first_start.kra");
+    if (!fname.isEmpty()) {
+        KoDocumentEntry entry = KoDocumentEntry(KoDocument::readNativeService());
+        QString errorMsg;
+        KoDocument* doc = entry.createDoc(&errorMsg);
+        if (!doc) return;
+        KoMainWindow *shell = new KoMainWindow(doc->componentData());
+        shell->show();
+        QObject::connect(doc, SIGNAL(sigProgress(int)), shell, SLOT(slotProgress(int)));
+        // for initDoc to fill in the recent docs list
+        // and for KoDocument::slotStarted
+        doc->addShell(shell);
+        doc->showStartUpWidget(shell);
+        doc->openUrl(fname);
+    }
+
+}
 
 #include "kis_view2.moc"
