@@ -23,11 +23,8 @@
 
 #include "KWOdfWriter.h"
 #include "KWDocument.h"
-#include <rdf/KoDocumentRdfBase.h>
 #include "KWPage.h"
 
-#include <ktemporaryfile.h>
-
 #include "frames/KWTextFrameSet.h"
 #include "frames/KWTextFrame.h"
 #include <KoXmlWriter.h>
@@ -44,10 +41,12 @@
 #include <KoTextSharedSavingData.h>
 
 #include <KoStoreDevice.h>
+#include <rdf/KoDocumentRdfBase.h>
 
 #include <QBuffer>
 #include <QTextCursor>
 #include <KDebug>
+#include <ktemporaryfile.h>
 
 QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver &embeddedSaver, KoGenStyles &mainStyles, KoGenChanges  &changes, KWTextFrameSet *fs)
 {
@@ -76,7 +75,7 @@ QByteArray KWOdfWriter::serializeHeaderFooter(KoEmbeddedDocumentSaver &embeddedS
     Q_ASSERT(shapedata);
 
     writer.startElement(tag);
-    shapedata->saveOdf(context, 0, -1, m_document->documentRdfBase());
+    shapedata->saveOdf(context, m_document->documentRdfBase());
     writer.endElement();
 
     return content;
@@ -241,8 +240,13 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
         //  3) frames that are not anchored but freely positioned somewhere on the page.
         //     in ODF terms those frames are page-anchored.
 
-        if (fs->frameCount() == 1 && fs->frames().first()->shape()->parent())
-            continue; // is a frame that is anchored to text, don't save those here.
+        if (fs->frameCount() == 1) {
+            KoShape *shape = fs->frames().first()->shape();
+            // may be a frame that is anchored to text, don't save those here.
+            // but first check since clipped shapes look similar, but are not anchored to text
+            if (shape->parent() && !shape->parent()->isClipped(shape))
+                continue;
+        }
 
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs) {
@@ -255,7 +259,6 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
             }
         }
 
-        KWFrame *lastNonCopyFrame = 0;
         int counter = 1;
         QSet<QString> uniqueNames;
         foreach (KWFrame *frame, fs->frames()) { // make sure all shapes have names.
@@ -269,89 +272,8 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
         const QList<KWFrame*> frames = fs->frames();
         for (int i = 0; i < frames.count(); ++i) {
             KWFrame *frame = frames.at(i);
-            KoShape *shape = frame->shape();
-            // frame properties first
-            shape->setAdditionalStyleAttribute("fo:margin", QString::number(frame->runAroundDistance()) + "pt");
-            shape->setAdditionalStyleAttribute("style:horizontal-pos", "from-left");
-            shape->setAdditionalStyleAttribute("style:horizontal-rel", "page");
-            shape->setAdditionalStyleAttribute("style:vertical-pos", "from-top");
-            shape->setAdditionalStyleAttribute("style:vertical-rel", "page");
-            QString value;
-            switch (frame->textRunAround()) {
-            case KWord::RunAround:
-                switch (frame->runAroundSide()) {
-                case KWord::BiggestRunAroundSide: value = "biggest"; break;
-                case KWord::LeftRunAroundSide: value = "left"; break;
-                case KWord::RightRunAroundSide: value = "right"; break;
-                case KWord::AutoRunAroundSide: value = "dynamic"; break;
-                case KWord::BothRunAroundSide: value = "parallel"; break;
-                }
-                break;
-            case KWord::RunThrough:
-                value = "run-through";
-                break;
-            case KWord::NoRunAround:
-                value = "none";
-                break;
-            }
-            shape->setAdditionalStyleAttribute("style:wrap", value);
-
-            switch (frame->frameBehavior()) {
-            case KWord::AutoCreateNewFrameBehavior:
-                value = "auto-create-new-frame";
-                break;
-            case KWord::IgnoreContentFrameBehavior:
-                value = "clip";
-                break;
-            case KWord::AutoExtendFrameBehavior:
-                // the third case, AutoExtendFrame is handled by min-height
-                value.clear();
-                KWTextFrame *tf = dynamic_cast<KWTextFrame*>(frame);
-                if (tf && tf->minimumFrameHeight() > 1)
-                    shape->setAdditionalAttribute("fo:min-height", QString::number(tf->minimumFrameHeight()) + "pt");
-                break;
-            }
-            if (!value.isEmpty())
-                shape->setAdditionalStyleAttribute("style:overflow-behavior", value);
-
-            if (frame->frameBehavior() != KWord::IgnoreContentFrameBehavior) {
-                switch (frame->newFrameBehavior()) {
-                case KWord::ReconnectNewFrame: value = "followup"; break;
-                case KWord::NoFollowupFrame: value.clear(); break; // "none" is the default
-                case KWord::CopyNewFrame: value = "copy"; break;
-                }
-                if (!value.isEmpty()) {
-                    shape->setAdditionalStyleAttribute("koffice:frame-behavior-on-new-page", value);
-                    if (! frame->frameOnBothSheets())
-                        shape->setAdditionalAttribute("koffice:frame-copy-to-facing-pages", "true");
-                }
-            }
-
-            if (frame->isCopy()) {
-                Q_ASSERT(lastNonCopyFrame);
-                shape->setAdditionalAttribute("draw:copy-of", lastNonCopyFrame->shape()->name());
-            } else {
-                lastNonCopyFrame = frame;
-            }
-
-            // shape properties
-            KWPage page = m_document->pageManager()->page(shape);
-            const qreal pagePos = page.offsetInDocument();
-
-            const int effectiveZIndex = shape->zIndex() + m_zIndexOffsets.value(page);
-            shape->setAdditionalAttribute("draw:z-index", QString::number(effectiveZIndex));
-            shape->setAdditionalAttribute("text:anchor-type", "page");
-            shape->setAdditionalAttribute("text:anchor-page-number", QString::number(page.pageNumber()));
-            context.addShapeOffset(shape, QMatrix(1, 0, 0 , 1, 0, -pagePos));
-            shape->saveOdf(context);
-            context.removeShapeOffset(shape);
-            shape->removeAdditionalAttribute("draw:copy-of");
-            shape->removeAdditionalAttribute("draw:z-index");
-            shape->removeAdditionalAttribute("fo:min-height");
-            shape->removeAdditionalAttribute("koffice:frame-copy-to-facing-pages");
-            shape->removeAdditionalAttribute("text:anchor-page-number");
-            shape->removeAdditionalAttribute("text:anchor-page-number");
-            shape->removeAdditionalAttribute("text:anchor-type");
+            KWPage page = m_document->pageManager()->page(frame->shape());
+            frame->saveOdf(context, page, m_zIndexOffsets.value(page));
         }
     }
 
@@ -367,7 +289,7 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
                     tbf.setProperty(KoParagraphStyle::MasterPageName, m_masterPages.value(style));
                     cursor.mergeBlockFormat(tbf);
                 }
-                shapeData->saveOdf(context, 0, -1, m_document->documentRdfBase());
+                shapeData->saveOdf(context, m_document->documentRdfBase());
             }
         }
     }
@@ -383,14 +305,20 @@ bool KWOdfWriter::save(KoOdfWriteStore &odfStore, KoEmbeddedDocumentSaver &embed
     tmpChangeFile.close();
     bodyWriter->addCompleteElement(&tmpChangeFile);
 
-    bodyWriter->startElement("text:page-sequence");
-    foreach (KWPage page, m_document->pageManager()->pages()) {
-        Q_ASSERT(m_masterPages.contains(page.pageStyle()));
-        bodyWriter->startElement("text:page");
-        bodyWriter->addAttribute("text:master-page-name", m_masterPages.value(page.pageStyle()));
-        bodyWriter->endElement(); // text:page
+    // Do not write out text:page-sequence, if there is a maintTextFrame
+    // The ODF specification does not allow text:page-sequence in office:text
+    // if there is e.g. text:p or text:h there
+    if (!mainTextFrame) {
+        bodyWriter->startElement("text:page-sequence");
+        foreach (KWPage page, m_document->pageManager()->pages()) {
+            Q_ASSERT(m_masterPages.contains(page.pageStyle()));
+            bodyWriter->startElement("text:page");
+            bodyWriter->addAttribute("text:master-page-name",
+                    m_masterPages.value(page.pageStyle()));
+            bodyWriter->endElement(); // text:page
+        }
+        bodyWriter->endElement(); // text:page-sequence
     }
-    bodyWriter->endElement(); // text:page-sequence
 
     delete tmpBodyWriter;
     tmpBodyWriter = 0;
@@ -466,7 +394,7 @@ void KWOdfWriter::calculateZindexOffsets()
 {
     Q_ASSERT(m_zIndexOffsets.isEmpty()); // call this method only once, please.
     foreach (KWFrameSet *fs, m_document->frameSets()) {
-        if (KWord::isAutoGenerated(dynamic_cast<KWTextFrameSet*>(fs)))
+        if (KWord::isAutoGenerated(fs))
             continue;
         foreach (KWFrame *frame, fs->frames()) {
             addShapeToTree(frame->shape());
@@ -490,7 +418,7 @@ void KWOdfWriter::addShapeToTree(KoShape *shape)
     // add the children of a KoShapeContainer
     KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(shape);
     if (container) {
-        foreach(KoShape *containerShape, container->childShapes()) {
+        foreach(KoShape *containerShape, container->shapes()) {
             addShapeToTree(containerShape);
         }
     }
