@@ -1,9 +1,13 @@
 /* This file is part of the KDE project
- * Copyright (C) 2004-2006 David Faure <faure@kde.org>
- * Copyright (C) 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2010 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2001-2006 David Faure <faure@kde.org>
  * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
  * Copyright (C) 2007 Pierre Ducroquet <pinaraf@gmail.com>
  * Copyright (C) 2007-2009 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
+ * Copyright (C) 2009 KO GmbH <cbo@kogmbh.com>
+ * Copyright (C) 2009 Pierre Stirnweiss <pstirnweiss@googlemail.com>
+ * Copyright (C) 2010 KO GmbH <ben.martin@kogmbh.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -30,6 +34,7 @@
 #include "KoXmlReaderForward.h"
 
 class KoShapeLoadingContext;
+class KoShape;
 class QTextCursor;
 class KoBookmarkManager;
 class KoDocumentRdfBase;
@@ -60,10 +65,12 @@ public:
 
     /**
     * Constructor.
+    * Notice that despite this being a QObject there is no 'parent' available for
+    * memory management here.
     *
     * @param context The context the KoTextLoader is called in
     */
-    explicit KoTextLoader(KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData = 0);
+    explicit KoTextLoader(KoShapeLoadingContext &context, KoDocumentRdfBase *rdfData = 0, KoShape *shape = 0);
 
     /**
     * Destructor.
@@ -108,6 +115,13 @@ private:
     */
     void loadSection(const KoXmlElement &element, QTextCursor &cursor);
 
+
+    /**
+    * Load the text into the \p cursor .
+    */
+    void loadText(const QString &text, QTextCursor &cursor,
+                  bool *stripLeadingSpace, bool isLastNode);
+
     /**
     * Load the span from the \p element into the \p cursor .
     */
