@@ -22,6 +22,7 @@
 
 #include <QtGui>
 #include <KDebug>
+#include <QTextStream>
 #include <QtScript>
 #include <QtTest>
 
@@ -144,9 +145,9 @@ bool TestLoading::compareFragments(const QTextFragment &actualFragment, const QT
                  && actualFormat.verticalAlignment() == expectedFormat.verticalAlignment(); // FIXME: Compare other properties
 
     if (!equal) {
-        qDebug() << "Actual property:   " << KoTextDebug::textAttributes(actualFormat);
-        qDebug() << "Expected property: " << KoTextDebug::textAttributes(expectedFormat);
-        qDebug() << "compareFragment: property mismatch at " << actualFragment.text();
+        qDebug() << "compareFragments: properties mismatch at " << actualFragment.text() << endl
+        << "actual:  " << KoTextDebug::textAttributes(actualFormat) << endl
+        << "expected:" << KoTextDebug::textAttributes(expectedFormat);
     }
 
     return equal;
@@ -236,8 +237,8 @@ bool TestLoading::compareBlockFormats(const QTextBlockFormat &actualFormat, cons
             break;
         }
         if (!match) {
-            qDebug() << "Actual property:   " << KoTextDebug::paraAttributes(actualFormat);
-            qDebug() << "Expected property: " << KoTextDebug::paraAttributes(expectedFormat);
+            qDebug() << "Actual:  " << KoTextDebug::paraAttributes(actualFormat);
+            qDebug() << "Expected:" << KoTextDebug::paraAttributes(expectedFormat);
             qDebug() << "At index: QTextFormat::UserProperty + " << id - QTextFormat::UserProperty;
             return false;
         }
@@ -249,15 +250,48 @@ bool TestLoading::compareListFormats(const QTextListFormat &actualFormat, const
 {
     QMap<int, QVariant> actualProperties = actualFormat.properties();
     actualProperties.remove(KoListStyle::StyleId);
-    actualProperties.remove(KoListStyle::Indent);
-    actualProperties.remove(KoListStyle::MinimumWidth);
     actualProperties.remove(KoListStyle::MinimumDistance);
+    actualProperties.remove(KoListStyle::ListId);
+    actualProperties.remove(QTextFormat::ObjectIndex);
 
     QMap<int, QVariant> expectedProperties = expectedFormat.properties();
     expectedProperties.remove(KoListStyle::StyleId);
-    expectedProperties.remove(KoListStyle::Indent);
-    expectedProperties.remove(KoListStyle::MinimumWidth);
     expectedProperties.remove(KoListStyle::MinimumDistance);
+    expectedProperties.remove(KoListStyle::ListId);
+    expectedProperties.remove(QTextFormat::ObjectIndex);
+
+
+    struct Enum {
+        static QString resolve(int key) {
+            QMetaObject meta;
+            if (key >= QTextFormat::UserProperty)
+                meta = KoListStyle::staticMetaObject;
+            else
+                meta = QTextFormat::staticMetaObject;
+            QMetaEnum me = meta.enumerator(meta.indexOfEnumerator("Property"));
+            QString answer(me.valueToKey(key));
+            if (answer.isEmpty())
+                answer = QString::number(key);
+            return answer;
+        }
+    };
+
+    foreach (int key, expectedProperties.keys()) {
+        if (actualProperties.contains(key)) {
+            if (actualProperties.value(key) != expectedProperties.value(key)) {
+                qDebug() << "properties don't match;" << Enum::resolve(key)
+                    << actualProperties.value(key) << expectedProperties.value(key);
+            }
+        } else {
+            qDebug() << "Expected but missing property;" << Enum::resolve(key);
+        }
+    }
+    foreach (int key, actualProperties.keys()) {
+        if (!expectedProperties.contains(key)) {
+            qDebug() << "found extra property;" << Enum::resolve(key);
+        }
+    }
+
     return actualProperties == expectedProperties;
 }
 
@@ -273,9 +307,11 @@ bool TestLoading::compareBlocks(const QTextBlock &actualBlock, const QTextBlock
         }
         if (!compareListFormats(actualList->format(), expectedList->format())
                 || (actualList->itemNumber(actualBlock) != expectedList->itemNumber(expectedBlock))) {
-            qDebug() << "compareBlocks: list properties mismatch at " << actualBlock.text()
-            << KoTextDebug::listAttributes(actualList->format()) << KoTextDebug::listAttributes(expectedList->format())
-            << actualList->itemNumber(actualBlock) << expectedList->itemNumber(expectedBlock);
+            qDebug() << "compareBlocks: list properties mismatch at " << actualBlock.text() << endl
+            << "actual:  " << KoTextDebug::listAttributes(actualList->format())
+            << actualList->itemNumber(actualBlock) << endl
+            << "expected:" << KoTextDebug::listAttributes(expectedList->format())
+            << expectedList->itemNumber(expectedBlock);
             return false;
         }
     } else {
@@ -315,8 +351,16 @@ bool TestLoading::compareBlocks(const QTextBlock &actualBlock, const QTextBlock
     }
 
     bool equal = actualIterator.atEnd() == expectedIterator.atEnd();
-    if (!equal)
-        qDebug() << "compareBlock: Iterator are not in the end! at " << actualBlock.text() << expectedBlock.text();
+    if (!equal) {
+        if (actualIterator.atEnd())
+            qDebug() << "compareBlock: expected more text after block length:"
+                << actualBlock.length() << ", expected:" << expectedBlock.length()
+                << "at position:" << actualBlock.position();
+        else
+            qDebug() << "compareBlock: more text than expected in block length:"
+                << actualBlock.length() << ", expected:" << expectedBlock.length()
+                << "at position:" << actualBlock.position();
+    }
 
     return equal;
 }
@@ -853,6 +897,10 @@ void TestLoading::cleanup()
 
 void TestLoading::addData(LoadSave loadSave)
 {
+    QTest::newRow("whitespace") << "TextContents/Paragraph/whitespace";
+    QTest::newRow("basicContents") << "TextContents/Paragraph/basicContents";
+    QTest::newRow("boldAndItalic") << "TextContents/TextFormatting/boldAndItalic";
+
     QTest::newRow("bulletedList") << "TextContents/Lists/bulletedList";
     QTest::newRow("continueNumbering") << "TextContents/Lists/continueNumbering";
     QTest::newRow("embeddedBulletedList") << "TextContents/Lists/embeddedBulletedList";
@@ -863,12 +911,9 @@ void TestLoading::addData(LoadSave loadSave)
     QTest::newRow("numberedParagraph") << "TextContents/Lists/numberedParagraph";
     QTest::newRow("startValue") << "TextContents/Lists/startValue";
 
-    QTest::newRow("boldAndItalic") << "TextContents/TextFormatting/boldAndItalic";
 
     QTest::newRow("attributedText") << "TextContents/Paragraph/attributedText";
-    QTest::newRow("basicContents") << "TextContents/Paragraph/basicContents";
     QTest::newRow("nestedSpan") << "TextContents/Paragraph/nestedSpan";
-    QTest::newRow("whitespace") << "TextContents/Paragraph/whitespace";
 
     QTest::newRow("fontSize1") << "TextContents/TextFormatting/fontSize";
 
@@ -1078,7 +1123,7 @@ QString TestLoading::documentToOdt(QTextDocument *document)
     KoChangeTracker* changeTracker = new KoChangeTracker;
     KoTextDocument(textShapeData->document()).setChangeTracker(changeTracker);
 
-    textShapeData->saveOdf(context);
+    textShapeData->saveOdf(context, 0);
 
     contentTmpFile.close();
 
@@ -1123,6 +1168,7 @@ void TestLoading::testLoading_data()
 void TestLoading::testLoading()
 {
     QFETCH(QString, testcase);
+    QString testName = testcase;
     testcase.prepend(FILES_DATA_DIR);
 
     QTextDocument *actualDocument = documentFromOdt(testcase + ".odt");
@@ -1137,9 +1183,22 @@ void TestLoading::testLoading()
 //    showDocument(actualDocument);
 //    showDocument(expectedDocument);
     if (!documentsEqual) {
-        KoTextDebug::dumpDocument(actualDocument);
-        KoTextDebug::dumpDocument(expectedDocument);
+        testName.replace('/', '_');
+        QFile file1("failed-loading-" + testName + "-actual");
+        if (file1.open(QFile::WriteOnly)) {
+            QTextStream out(&file1);
+            KoTextDebug::dumpDocument(actualDocument, out);
+            file1.close();
+        }
+        QFile file2("failed-loading-" + testName + "-expected");
+        if (file2.open(QFile::WriteOnly)) {
+            QTextStream out(&file2);
+            KoTextDebug::dumpDocument(expectedDocument, out);
+            file2.close();
+        }
     }
+    if (testcase.indexOf("Tables/") > 0)
+        QEXPECT_FAIL("", "Tables not supported yet", Abort);
     delete actualDocument;
     delete expectedDocument;
     QVERIFY(documentsEqual);
@@ -1155,6 +1214,7 @@ void TestLoading::testSaving_data()
 void TestLoading::testSaving()
 {
     QFETCH(QString, testcase);
+    QString testName = testcase;
     testcase.prepend(FILES_DATA_DIR);
 
     QTextDocument *actualDocument = documentFromOdt(testcase + ".odt");
@@ -1169,12 +1229,25 @@ void TestLoading::testSaving()
     bool documentsEqual = compareDocuments(savedDocument, expectedDocument);
 
     if (!documentsEqual) {
-        KoTextDebug::dumpDocument(savedDocument);
-        KoTextDebug::dumpDocument(expectedDocument);
+        testName.replace('/', '_');
+        QFile file1("failed-saving-" + testName + "-actual");
+        if (file1.open(QFile::WriteOnly)) {
+            QTextStream out(&file1);
+            KoTextDebug::dumpDocument(savedDocument, out);
+            file1.close();
+        }
+        QFile file2("failed-saving-" + testName + "-expected");
+        if (file2.open(QFile::WriteOnly)) {
+            QTextStream out(&file2);
+            KoTextDebug::dumpDocument(expectedDocument, out);
+            file2.close();
+        }
     }
     delete actualDocument;
     delete expectedDocument;
     delete savedDocument;
+    if (testcase.indexOf("Tables/") > 0)
+        QEXPECT_FAIL("", "Tables not supported yet", Abort);
     QVERIFY(documentsEqual);
 }
 
