@@ -2,6 +2,7 @@
    Copyright (C) 1998, 1999 Torben Weis <weis@kde.org>
    Copyright (C) 2000-2006 David Faure <faure@kde.org>
    Copyright (C) 2007, 2009 Thomas zander <zander@kde.org>
+   Copyright (C) 2010 Benjamin Port <port.benjamin@gmail.com>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -197,6 +198,7 @@ public:
     KAction *showFileVersions;
     KAction *importFile;
     KAction *exportFile;
+    KToggleAction *toggleDockers;
     KRecentFilesAction *recent;
 
     bool isImporting;
@@ -338,6 +340,14 @@ KoMainWindow::KoMainWindow(const KComponentData &componentData)
     sep->setSeparator(true);
     d->splitViewActionList.append(sep);
 
+    d->toggleDockers = new KToggleAction(i18n("Show Dockers"), this);
+    d->toggleDockers->setCheckedState(KGuiItem(i18n("Hide Dockers")));
+    d->toggleDockers->setChecked(true);
+    actionCollection()->addAction("view_toggledockers", d->toggleDockers);
+
+    d->toggleDockers->setShortcut(QKeySequence(Qt::CTRL + Qt::Key_H));
+    connect(d->toggleDockers, SIGNAL(toggled(bool)), SLOT(toggleDockersVisibility(bool)));
+
     d->dockWidgetMenu  = new KActionMenu(i18n("Dockers"), this);
     actionCollection()->addAction("settings_dockers_menu", d->dockWidgetMenu);
     d->dockWidgetMenu->setVisible(false);
@@ -423,10 +433,8 @@ void KoMainWindow::setRootDocument(KoDocument *doc)
     if (d->docToOpen && d->docToOpen != doc) {
         d->docToOpen->removeShell(this);
         delete d->docToOpen;
-        d->docToOpen = 0;
-    } else {
-        d->docToOpen = 0;
     }
+    d->docToOpen = 0;
 
     //kDebug(30003) <<"KoMainWindow::setRootDocument this =" << this <<" doc =" << doc;
     QList<KoView*> oldRootViews = d->rootViews;
@@ -963,6 +971,7 @@ bool KoMainWindow::saveDocument(bool saveas, bool silent)
                     if (ret) {
                         kDebug(30003) << "Successful Save As!";
                         addRecentURL(newURL);
+                        setReadWrite(true);
                     } else {
                         kDebug(30003) << "Failed Save As!";
                         pDoc->setUrl(oldURL);
@@ -1323,11 +1332,18 @@ KoPrintJob* KoMainWindow::exportToPdf(QString pdfFileName)
         if (dialog.exec() != QDialog::Accepted)
             return 0;
         KUrl url(dialog.selectedUrl());
-        // TODO warn when overwriting
+        if (KIO::NetAccess::exists(url,  KIO::NetAccess::DestinationSide, this)) {
+            bool overwrite = KMessageBox::questionYesNo(this,
+                                            i18n("A document with this name already exists.\n"\
+                                                "Do you want to overwrite it?"),
+                                            i18n("Warning")) == KMessageBox::Yes;
+            if (!overwrite) {
+                return 0;
+            }
+        }
         pdfFileName = url.toLocalFile();
     }
-
-    KoPrintJob *printJob = rootView()->createPrintJob();
+    KoPrintJob *printJob = rootView()->createPdfPrintJob();
     if (printJob == 0)
         return 0;
     d->applyDefaultSettings(printJob->printer());
@@ -1349,7 +1365,7 @@ void KoMainWindow::slotConfigureToolbars()
     if (rootDocument())
         saveMainWindowSettings(KGlobal::config()->group(rootDocument()->componentData().componentName()));
     KEditToolBar edit(factory(), this);
-    connect(&edit, SIGNAL(newToolbarConfig()), this, SLOT(slotNewToolbarConfig()));
+    connect(&edit, SIGNAL(newToolBarConfig()), this, SLOT(slotNewToolbarConfig()));
     (void) edit.exec();
 }
 
@@ -1466,9 +1482,6 @@ void KoMainWindow::slotRemoveView()
         kWarning() << "view not found in d->rootViews!";
     }
 
-    // Prevent the view's destroyed() signal from triggering GUI rebuilding (too early)
-    d->manager->setActivePart(0, 0);
-
     d->rootViews.removeAll(view);
     delete view;
     view = 0;
@@ -1514,7 +1527,7 @@ void KoMainWindow::slotProgress(int value)
     if (d->firstTime) {
         // The statusbar might not even be created yet.
         // So check for that first, and create it if necessary
-        QStatusBar* bar = qFindChild<QStatusBar *>(this);
+        QStatusBar *bar = findChild<QStatusBar *>();
         if (!bar) {
             statusBar()->show();
             QApplication::sendPostedEvents(this, QEvent::ChildAdded);
@@ -1775,11 +1788,10 @@ QDockWidget* KoMainWindow::createDockWidget(KoDockFactoryBase* factory)
 
         KoDockWidgetTitleBar *titleBar = 0;
         // Check if the dock widget is supposed to be collapsable
-        if (factory->isCollapsable()) {
-            if (!dockWidget->titleBarWidget()) {
-                titleBar = new KoDockWidgetTitleBar(dockWidget);
-                dockWidget->setTitleBarWidget(titleBar);
-            }
+        if (!dockWidget->titleBarWidget()) {
+            titleBar = new KoDockWidgetTitleBar(dockWidget);
+            dockWidget->setTitleBarWidget(titleBar);
+            titleBar->setCollapsable(factory->isCollapsable());
         }
 
         dockWidget->setObjectName(factory->id());
@@ -1889,7 +1901,6 @@ KoDockerManager * KoMainWindow::dockerManager() const
 void KoMainWindow::toggleDockersVisibility(bool v) const
 {
     Q_UNUSED(v);
-    qDebug() << "toggleDockersVisibility";
     if (d->hiddenDockwidgets.isEmpty()){
         foreach(QObject* widget, children()) {
             if (widget->inherits("QDockWidget")) {
